import typing, clr, abc
from System import ValueTuple_2, Predicate_1, ValueTuple_3, Func_2, Action_1, Version, TimeSpan, EventHandler, IEquatable_1, IAsyncResult, Delegate, Array_1
from System.IO import DirectoryInfo, FileInfo
from System.Collections.ObjectModel import ObservableCollection_1, Collection_1
from System.Collections.Generic import IEnumerable_1, List_1, IReadOnlyCollection_1, IList_1
from SIMULTAN.Data.SimMath import SimPoint3D, SimVector3D, SimMatrix3D, SimPoint, SimVector, SimQuaternion
from System.Collections import IList
from System.Collections.Specialized import NotifyCollectionChangedEventArgs
from System.Security import SecureString
from SIMULTAN.Data.Taxonomy import SimTaxonomy, SimTaxonomyEntryOrString
from System.Globalization import CultureInfo
from System.ComponentModel import ISynchronizeInvoke

class AdmissibilityQueries(abc.ABC):
    @staticmethod
    def DirectoryNameIsAdmissible(new_dir: DirectoryInfo, isAdmissible: Predicate_1[str], collisionFormat: str) -> ValueTuple_2[bool, str]: ...
    @staticmethod
    def FileNameIsAdmissible(new_file: FileInfo, isAdmissible: Predicate_1[str], collisionFormat: str) -> ValueTuple_2[bool, str]: ...
    @staticmethod
    def FindCopyName(name: str, isUsed: Predicate_1[str], copyFormat: str, copyCollisionFormat: str) -> str: ...
    @staticmethod
    def PropertyNameIsAdmissible(name: str, isAdmissible: Predicate_1[str], collisionNameFormat: str) -> ValueTuple_2[bool, str]: ...


class CollectionExtensions(abc.ABC):
    # Skipped AddRange due to it being static, abstract and generic.

    AddRange : AddRange_MethodGroup
    class AddRange_MethodGroup:
        def __getitem__(self, t:typing.Type[AddRange_1_T1]) -> AddRange_1[AddRange_1_T1]: ...

        AddRange_1_T1 = typing.TypeVar('AddRange_1_T1')
        class AddRange_1(typing.Generic[AddRange_1_T1]):
            AddRange_1_T = CollectionExtensions.AddRange_MethodGroup.AddRange_1_T1
            def __call__(self, collection: ObservableCollection_1[AddRange_1_T], items: IEnumerable_1[AddRange_1_T]) -> None:...


    # Skipped ArgMax due to it being static, abstract and generic.

    ArgMax : ArgMax_MethodGroup
    class ArgMax_MethodGroup:
        def __getitem__(self, t:typing.Tuple[typing.Type[ArgMax_2_T1], typing.Type[ArgMax_2_T2]]) -> ArgMax_2[ArgMax_2_T1, ArgMax_2_T2]: ...

        ArgMax_2_T1 = typing.TypeVar('ArgMax_2_T1')
        ArgMax_2_T2 = typing.TypeVar('ArgMax_2_T2')
        class ArgMax_2(typing.Generic[ArgMax_2_T1, ArgMax_2_T2]):
            ArgMax_2_T = CollectionExtensions.ArgMax_MethodGroup.ArgMax_2_T1
            ArgMax_2_Key = CollectionExtensions.ArgMax_MethodGroup.ArgMax_2_T2
            def __call__(self, collection: IEnumerable_1[ArgMax_2_T], key: Func_2[ArgMax_2_T, ArgMax_2_Key]) -> ValueTuple_3[ArgMax_2_T, ArgMax_2_Key, int]:...


    # Skipped ArgMin due to it being static, abstract and generic.

    ArgMin : ArgMin_MethodGroup
    class ArgMin_MethodGroup:
        def __getitem__(self, t:typing.Tuple[typing.Type[ArgMin_2_T1], typing.Type[ArgMin_2_T2]]) -> ArgMin_2[ArgMin_2_T1, ArgMin_2_T2]: ...

        ArgMin_2_T1 = typing.TypeVar('ArgMin_2_T1')
        ArgMin_2_T2 = typing.TypeVar('ArgMin_2_T2')
        class ArgMin_2(typing.Generic[ArgMin_2_T1, ArgMin_2_T2]):
            ArgMin_2_T = CollectionExtensions.ArgMin_MethodGroup.ArgMin_2_T1
            ArgMin_2_Key = CollectionExtensions.ArgMin_MethodGroup.ArgMin_2_T2
            def __call__(self, collection: IEnumerable_1[ArgMin_2_T], key: Func_2[ArgMin_2_T, ArgMin_2_Key]) -> ValueTuple_3[ArgMin_2_T, ArgMin_2_Key, int]:...


    # Skipped Average due to it being static, abstract and generic.

    Average : Average_MethodGroup
    class Average_MethodGroup:
        def __getitem__(self, t:typing.Type[Average_1_T1]) -> Average_1[Average_1_T1]: ...

        Average_1_T1 = typing.TypeVar('Average_1_T1')
        class Average_1(typing.Generic[Average_1_T1]):
            Average_1_T = CollectionExtensions.Average_MethodGroup.Average_1_T1
            @typing.overload
            def __call__(self, enumerable: IEnumerable_1[Average_1_T], selector: Func_2[Average_1_T, SimPoint3D]) -> SimPoint3D:...
            @typing.overload
            def __call__(self, enumerable: IEnumerable_1[Average_1_T], selector: Func_2[Average_1_T, SimVector3D]) -> SimVector3D:...


    # Skipped DeepCopy due to it being static, abstract and generic.

    DeepCopy : DeepCopy_MethodGroup
    class DeepCopy_MethodGroup:
        def __getitem__(self, t:typing.Type[DeepCopy_1_T1]) -> DeepCopy_1[DeepCopy_1_T1]: ...

        DeepCopy_1_T1 = typing.TypeVar('DeepCopy_1_T1')
        class DeepCopy_1(typing.Generic[DeepCopy_1_T1]):
            DeepCopy_1_T = CollectionExtensions.DeepCopy_MethodGroup.DeepCopy_1_T1
            def __call__(self, source: List_1[List_1[DeepCopy_1_T]]) -> List_1[List_1[DeepCopy_1_T]]:...


    # Skipped DistinctBy due to it being static, abstract and generic.

    DistinctBy : DistinctBy_MethodGroup
    class DistinctBy_MethodGroup:
        def __getitem__(self, t:typing.Type[DistinctBy_1_T1]) -> DistinctBy_1[DistinctBy_1_T1]: ...

        DistinctBy_1_T1 = typing.TypeVar('DistinctBy_1_T1')
        class DistinctBy_1(typing.Generic[DistinctBy_1_T1]):
            DistinctBy_1_T = CollectionExtensions.DistinctBy_MethodGroup.DistinctBy_1_T1
            def __call__(self, enumerable: IEnumerable_1[DistinctBy_1_T], selector: Func_2[DistinctBy_1_T, typing.Any]) -> IEnumerable_1[DistinctBy_1_T]:...


    # Skipped FindIndex due to it being static, abstract and generic.

    FindIndex : FindIndex_MethodGroup
    class FindIndex_MethodGroup:
        def __getitem__(self, t:typing.Type[FindIndex_1_T1]) -> FindIndex_1[FindIndex_1_T1]: ...

        FindIndex_1_T1 = typing.TypeVar('FindIndex_1_T1')
        class FindIndex_1(typing.Generic[FindIndex_1_T1]):
            FindIndex_1_T = CollectionExtensions.FindIndex_MethodGroup.FindIndex_1_T1
            def __call__(self, list: IEnumerable_1[FindIndex_1_T], predicate: Predicate_1[FindIndex_1_T]) -> int:...


    # Skipped ForEach due to it being static, abstract and generic.

    ForEach : ForEach_MethodGroup
    class ForEach_MethodGroup:
        def __getitem__(self, t:typing.Type[ForEach_1_T1]) -> ForEach_1[ForEach_1_T1]: ...

        ForEach_1_T1 = typing.TypeVar('ForEach_1_T1')
        class ForEach_1(typing.Generic[ForEach_1_T1]):
            ForEach_1_T = CollectionExtensions.ForEach_MethodGroup.ForEach_1_T1
            @typing.overload
            def __call__(self, collection: ObservableCollection_1[ForEach_1_T], action: Action_1[ForEach_1_T]) -> None:...
            @typing.overload
            def __call__(self, collection: IReadOnlyCollection_1[ForEach_1_T], action: Action_1[ForEach_1_T]) -> None:...
            @typing.overload
            def __call__(self, collection: IEnumerable_1[ForEach_1_T], action: Action_1[ForEach_1_T]) -> None:...

        def __call__(self, collection: IList, action: Action_1[typing.Any]) -> None:...

    # Skipped HandleCollectionChanged due to it being static, abstract and generic.

    HandleCollectionChanged : HandleCollectionChanged_MethodGroup
    class HandleCollectionChanged_MethodGroup:
        def __getitem__(self, t:typing.Tuple[typing.Type[HandleCollectionChanged_2_T1], typing.Type[HandleCollectionChanged_2_T2]]) -> HandleCollectionChanged_2[HandleCollectionChanged_2_T1, HandleCollectionChanged_2_T2]: ...

        HandleCollectionChanged_2_T1 = typing.TypeVar('HandleCollectionChanged_2_T1')
        HandleCollectionChanged_2_T2 = typing.TypeVar('HandleCollectionChanged_2_T2')
        class HandleCollectionChanged_2(typing.Generic[HandleCollectionChanged_2_T1, HandleCollectionChanged_2_T2]):
            HandleCollectionChanged_2_TSource = CollectionExtensions.HandleCollectionChanged_MethodGroup.HandleCollectionChanged_2_T1
            HandleCollectionChanged_2_TTarget = CollectionExtensions.HandleCollectionChanged_MethodGroup.HandleCollectionChanged_2_T2
            def __call__(self, args: NotifyCollectionChangedEventArgs, target: IList_1[HandleCollectionChanged_2_TTarget], convert: Func_2[HandleCollectionChanged_2_TSource, HandleCollectionChanged_2_TTarget]) -> None:...


    # Skipped IndicesWhere due to it being static, abstract and generic.

    IndicesWhere : IndicesWhere_MethodGroup
    class IndicesWhere_MethodGroup:
        def __getitem__(self, t:typing.Type[IndicesWhere_1_T1]) -> IndicesWhere_1[IndicesWhere_1_T1]: ...

        IndicesWhere_1_T1 = typing.TypeVar('IndicesWhere_1_T1')
        class IndicesWhere_1(typing.Generic[IndicesWhere_1_T1]):
            IndicesWhere_1_T = CollectionExtensions.IndicesWhere_MethodGroup.IndicesWhere_1_T1
            def __call__(self, source: IEnumerable_1[IndicesWhere_1_T], predicate: Func_2[IndicesWhere_1_T, bool]) -> IEnumerable_1[int]:...


    # Skipped RemoveFirst due to it being static, abstract and generic.

    RemoveFirst : RemoveFirst_MethodGroup
    class RemoveFirst_MethodGroup:
        def __getitem__(self, t:typing.Type[RemoveFirst_1_T1]) -> RemoveFirst_1[RemoveFirst_1_T1]: ...

        RemoveFirst_1_T1 = typing.TypeVar('RemoveFirst_1_T1')
        class RemoveFirst_1(typing.Generic[RemoveFirst_1_T1]):
            RemoveFirst_1_T = CollectionExtensions.RemoveFirst_MethodGroup.RemoveFirst_1_T1
            @typing.overload
            def __call__(self, collection: Collection_1[RemoveFirst_1_T], predicate: Func_2[RemoveFirst_1_T, bool]) -> bool:...
            @typing.overload
            def __call__(self, collection: Collection_1[RemoveFirst_1_T], predicate: Func_2[RemoveFirst_1_T, bool], removed: clr.Reference[RemoveFirst_1_T]) -> bool:...


    # Skipped RemoveWhere due to it being static, abstract and generic.

    RemoveWhere : RemoveWhere_MethodGroup
    class RemoveWhere_MethodGroup:
        def __getitem__(self, t:typing.Type[RemoveWhere_1_T1]) -> RemoveWhere_1[RemoveWhere_1_T1]: ...

        RemoveWhere_1_T1 = typing.TypeVar('RemoveWhere_1_T1')
        class RemoveWhere_1(typing.Generic[RemoveWhere_1_T1]):
            RemoveWhere_1_T = CollectionExtensions.RemoveWhere_MethodGroup.RemoveWhere_1_T1
            @typing.overload
            def __call__(self, collection: Collection_1[RemoveWhere_1_T], predicate: Func_2[RemoveWhere_1_T, bool]) -> None:...
            @typing.overload
            def __call__(self, collection: IList_1[RemoveWhere_1_T], predicate: Func_2[RemoveWhere_1_T, bool]) -> None:...
            @typing.overload
            def __call__(self, collection: Collection_1[RemoveWhere_1_T], predicate: Func_2[RemoveWhere_1_T, bool], removeAction: Action_1[RemoveWhere_1_T]) -> None:...
            @typing.overload
            def __call__(self, collection: IList_1[RemoveWhere_1_T], predicate: Func_2[RemoveWhere_1_T, bool], removeAction: Action_1[RemoveWhere_1_T]) -> None:...


    # Skipped Split due to it being static, abstract and generic.

    Split : Split_MethodGroup
    class Split_MethodGroup:
        def __getitem__(self, t:typing.Type[Split_1_T1]) -> Split_1[Split_1_T1]: ...

        Split_1_T1 = typing.TypeVar('Split_1_T1')
        class Split_1(typing.Generic[Split_1_T1]):
            Split_1_T = CollectionExtensions.Split_MethodGroup.Split_1_T1
            def __call__(self, list: IEnumerable_1[Split_1_T], predicate: Predicate_1[Split_1_T]) -> ValueTuple_2[IEnumerable_1[Split_1_T], IEnumerable_1[Split_1_T]]:...


    # Skipped ToList due to it being static, abstract and generic.

    ToList : ToList_MethodGroup
    class ToList_MethodGroup:
        def __getitem__(self, t:typing.Type[ToList_1_T1]) -> ToList_1[ToList_1_T1]: ...

        ToList_1_T1 = typing.TypeVar('ToList_1_T1')
        class ToList_1(typing.Generic[ToList_1_T1]):
            ToList_1_T = CollectionExtensions.ToList_MethodGroup.ToList_1_T1
            def __call__(self, iList: IList) -> IList_1[ToList_1_T]:...


    # Skipped ToObservableCollection due to it being static, abstract and generic.

    ToObservableCollection : ToObservableCollection_MethodGroup
    class ToObservableCollection_MethodGroup:
        def __getitem__(self, t:typing.Type[ToObservableCollection_1_T1]) -> ToObservableCollection_1[ToObservableCollection_1_T1]: ...

        ToObservableCollection_1_T1 = typing.TypeVar('ToObservableCollection_1_T1')
        class ToObservableCollection_1(typing.Generic[ToObservableCollection_1_T1]):
            ToObservableCollection_1_T = CollectionExtensions.ToObservableCollection_MethodGroup.ToObservableCollection_1_T1
            def __call__(self, enumerable: IEnumerable_1[ToObservableCollection_1_T]) -> ObservableCollection_1[ToObservableCollection_1_T]:...


    # Skipped Transpose due to it being static, abstract and generic.

    Transpose : Transpose_MethodGroup
    class Transpose_MethodGroup:
        def __getitem__(self, t:typing.Type[Transpose_1_T1]) -> Transpose_1[Transpose_1_T1]: ...

        Transpose_1_T1 = typing.TypeVar('Transpose_1_T1')
        class Transpose_1(typing.Generic[Transpose_1_T1]):
            Transpose_1_T = CollectionExtensions.Transpose_MethodGroup.Transpose_1_T1
            def __call__(self, input: List_1[List_1[Transpose_1_T]]) -> List_1[List_1[Transpose_1_T]]:...


    # Skipped TryFirstOrDefault due to it being static, abstract and generic.

    TryFirstOrDefault : TryFirstOrDefault_MethodGroup
    class TryFirstOrDefault_MethodGroup:
        def __getitem__(self, t:typing.Type[TryFirstOrDefault_1_T1]) -> TryFirstOrDefault_1[TryFirstOrDefault_1_T1]: ...

        TryFirstOrDefault_1_T1 = typing.TypeVar('TryFirstOrDefault_1_T1')
        class TryFirstOrDefault_1(typing.Generic[TryFirstOrDefault_1_T1]):
            TryFirstOrDefault_1_T = CollectionExtensions.TryFirstOrDefault_MethodGroup.TryFirstOrDefault_1_T1
            def __call__(self, source: IEnumerable_1[TryFirstOrDefault_1_T], predicate: Predicate_1[TryFirstOrDefault_1_T], value: clr.Reference[TryFirstOrDefault_1_T]) -> bool:...


    # Skipped TryGetElementAt due to it being static, abstract and generic.

    TryGetElementAt : TryGetElementAt_MethodGroup
    class TryGetElementAt_MethodGroup:
        def __getitem__(self, t:typing.Type[TryGetElementAt_1_T1]) -> TryGetElementAt_1[TryGetElementAt_1_T1]: ...

        TryGetElementAt_1_T1 = typing.TypeVar('TryGetElementAt_1_T1')
        class TryGetElementAt_1(typing.Generic[TryGetElementAt_1_T1]):
            TryGetElementAt_1_T = CollectionExtensions.TryGetElementAt_MethodGroup.TryGetElementAt_1_T1
            def __call__(self, list: IEnumerable_1[TryGetElementAt_1_T], index: int, result: clr.Reference[TryGetElementAt_1_T]) -> bool:...




class CommonExtensions(abc.ABC):
    @staticmethod
    def ConvertToDoubleIfNumeric(value: typing.Any) -> float: ...
    @staticmethod
    def EqualsWithNan(d: float, other: float) -> bool: ...
    @staticmethod
    def GetVersion(version: Version, fieldCount: int) -> Version: ...
    @staticmethod
    def IsCollinear(v1: SimVector3D, v2: SimVector3D, threshold: float = ...) -> bool: ...
    @staticmethod
    def MatrixFromAxes(x: SimVector3D, y: SimVector3D, z: SimVector3D, p: SimPoint3D) -> SimMatrix3D: ...
    @staticmethod
    def SecureEquals(s1: SecureString, s2: SecureString) -> bool: ...
    @staticmethod
    def Transpose(m: SimMatrix3D) -> SimMatrix3D: ...
    # Skipped Clamp due to it being static, abstract and generic.

    Clamp : Clamp_MethodGroup
    class Clamp_MethodGroup:
        def __getitem__(self, t:typing.Type[Clamp_1_T1]) -> Clamp_1[Clamp_1_T1]: ...

        Clamp_1_T1 = typing.TypeVar('Clamp_1_T1')
        class Clamp_1(typing.Generic[Clamp_1_T1]):
            Clamp_1_T = CommonExtensions.Clamp_MethodGroup.Clamp_1_T1
            def __call__(self, val: Clamp_1_T, min: Clamp_1_T, max: Clamp_1_T) -> Clamp_1_T:...


    # Skipped InRange due to it being static, abstract and generic.

    InRange : InRange_MethodGroup
    class InRange_MethodGroup:
        def __getitem__(self, t:typing.Type[InRange_1_T1]) -> InRange_1[InRange_1_T1]: ...

        InRange_1_T1 = typing.TypeVar('InRange_1_T1')
        class InRange_1(typing.Generic[InRange_1_T1]):
            InRange_1_T = CommonExtensions.InRange_MethodGroup.InRange_1_T1
            def __call__(self, value: InRange_1_T, minRange: InRange_1_T, maxRange: InRange_1_T) -> bool:...




class IDispatcherTimer(typing.Protocol):
    @property
    def Interval(self) -> TimeSpan: ...
    @Interval.setter
    def Interval(self, value: TimeSpan) -> TimeSpan: ...
    @abc.abstractmethod
    def AddTickEventHandler(self, handler: EventHandler) -> None: ...
    @abc.abstractmethod
    def RemoveTickEventHandler(self, handler: EventHandler) -> None: ...
    @abc.abstractmethod
    def Start(self) -> None: ...
    @abc.abstractmethod
    def Stop(self) -> None: ...


class IDispatcherTimerFactory(typing.Protocol):
    @abc.abstractmethod
    def Create(self) -> IDispatcherTimer: ...


class IntIndex2D(IEquatable_1[IntIndex2D]):
    def __init__(self, x: int, y: int) -> None: ...
    @property
    def Item(self) -> int: ...
    @Item.setter
    def Item(self, value: int) -> int: ...
    @property
    def X(self) -> int: ...
    @X.setter
    def X(self, value: int) -> int: ...
    @property
    def Y(self) -> int: ...
    @Y.setter
    def Y(self, value: int) -> int: ...
    def GetHashCode(self) -> int: ...
    def __add__(self, lhs: IntIndex2D, rhs: IntIndex2D) -> IntIndex2D: ...
    def __eq__(self, lhs: IntIndex2D, rhs: IntIndex2D) -> bool: ...
    def __ne__(self, lhs: IntIndex2D, rhs: IntIndex2D) -> bool: ...
    def __sub__(self, lhs: IntIndex2D, rhs: IntIndex2D) -> IntIndex2D: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, other: IntIndex2D) -> bool:...
        @typing.overload
        def __call__(self, obj: typing.Any) -> bool:...



class IntIndex3D(IEquatable_1[IntIndex3D]):
    def __init__(self, x: int, y: int, z: int) -> None: ...
    @property
    def Item(self) -> int: ...
    @Item.setter
    def Item(self, value: int) -> int: ...
    @property
    def X(self) -> int: ...
    @X.setter
    def X(self, value: int) -> int: ...
    @property
    def Y(self) -> int: ...
    @Y.setter
    def Y(self, value: int) -> int: ...
    @property
    def Z(self) -> int: ...
    @Z.setter
    def Z(self, value: int) -> int: ...
    def Equals(self, other: IntIndex3D) -> bool: ...
    def GetHashCode(self) -> int: ...


class IServicesProvider(typing.Protocol):
    # Skipped GetService due to it being static, abstract and generic.

    GetService : GetService_MethodGroup
    class GetService_MethodGroup:
        def __getitem__(self, t:typing.Type[GetService_1_T1]) -> GetService_1[GetService_1_T1]: ...

        GetService_1_T1 = typing.TypeVar('GetService_1_T1')
        class GetService_1(typing.Generic[GetService_1_T1]):
            GetService_1_T = IServicesProvider.GetService_MethodGroup.GetService_1_T1
            def __call__(self) -> GetService_1_T:...




class PointVectorExtensions(abc.ABC):
    @staticmethod
    def At(p: SimPoint3D, idx: int) -> float: ...
    @staticmethod
    def Divide(p: SimPoint3D, d: float) -> SimPoint3D: ...
    @staticmethod
    def Multiply(p: SimPoint3D, d: float) -> SimPoint3D: ...
    # Skipped Get due to it being static, abstract and generic.

    Get : Get_MethodGroup
    class Get_MethodGroup:
        @typing.overload
        def __call__(self, v: SimVector3D, idx: int) -> float:...
        @typing.overload
        def __call__(self, v: SimPoint3D, idx: int) -> float:...

    # Skipped XY due to it being static, abstract and generic.

    XY : XY_MethodGroup
    class XY_MethodGroup:
        @typing.overload
        def __call__(self, p: SimPoint3D) -> SimPoint:...
        @typing.overload
        def __call__(self, v: SimVector3D) -> SimVector:...



class Range3D:
    @typing.overload
    def __init__(self, minimum: SimPoint3D, maximum: SimPoint3D) -> None: ...
    @typing.overload
    def __init__(self, original: Range3D) -> None: ...
    @property
    def Maximum(self) -> SimPoint3D: ...
    @property
    def Minimum(self) -> SimPoint3D: ...
    # Skipped Contains due to it being static, abstract and generic.

    Contains : Contains_MethodGroup
    class Contains_MethodGroup:
        @typing.overload
        def __call__(self, position: SimPoint3D) -> bool:...
        @typing.overload
        def __call__(self, position: IntIndex3D) -> bool:...



class RowColumnIndex(IEquatable_1[RowColumnIndex]):
    def __init__(self, row: int, column: int) -> None: ...
    @property
    def Column(self) -> int: ...
    @Column.setter
    def Column(self, value: int) -> int: ...
    @property
    def Row(self) -> int: ...
    @Row.setter
    def Row(self, value: int) -> int: ...
    @staticmethod
    def FromIndex2D(index: IntIndex2D) -> RowColumnIndex: ...
    def GetHashCode(self) -> int: ...
    def __add__(self, lhs: RowColumnIndex, rhs: RowColumnIndex) -> RowColumnIndex: ...
    def __eq__(self, lhs: RowColumnIndex, rhs: RowColumnIndex) -> bool: ...
    def __ne__(self, lhs: RowColumnIndex, rhs: RowColumnIndex) -> bool: ...
    def __sub__(self, lhs: RowColumnIndex, rhs: RowColumnIndex) -> RowColumnIndex: ...
    def ToIndex2D(self) -> IntIndex2D: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, other: RowColumnIndex) -> bool:...
        @typing.overload
        def __call__(self, obj: typing.Any) -> bool:...



class RowColumnRange(IEquatable_1[RowColumnRange]):
    def __init__(self, rowStart: int, columnStart: int, rowCount: int, columnCount: int) -> None: ...
    @property
    def ColumnCount(self) -> int: ...
    @property
    def ColumnStart(self) -> int: ...
    @property
    def RowCount(self) -> int: ...
    @property
    def RowStart(self) -> int: ...
    def GetHashCode(self) -> int: ...
    def __eq__(self, lhs: RowColumnRange, rhs: RowColumnRange) -> bool: ...
    def __ne__(self, lhs: RowColumnRange, rhs: RowColumnRange) -> bool: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, other: RowColumnRange) -> bool:...
        @typing.overload
        def __call__(self, obj: typing.Any) -> bool:...

    # Skipped Merge due to it being static, abstract and generic.

    Merge : Merge_MethodGroup
    class Merge_MethodGroup:
        @typing.overload
        def __call__(self, lhs: RowColumnRange, rhs: RowColumnRange) -> RowColumnRange:...
        @typing.overload
        def __call__(self, lhs: RowColumnRange, rhs: RowColumnIndex) -> RowColumnRange:...



class ServicesProvider(IServicesProvider):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, serviceProvider: IServicesProvider) -> None: ...
    # Skipped AddService due to it being static, abstract and generic.

    AddService : AddService_MethodGroup
    class AddService_MethodGroup:
        def __getitem__(self, t:typing.Type[AddService_1_T1]) -> AddService_1[AddService_1_T1]: ...

        AddService_1_T1 = typing.TypeVar('AddService_1_T1')
        class AddService_1(typing.Generic[AddService_1_T1]):
            AddService_1_TServiceScope = ServicesProvider.AddService_MethodGroup.AddService_1_T1
            def __call__(self, service: AddService_1_TServiceScope) -> None:...


    # Skipped GetService due to it being static, abstract and generic.

    GetService : GetService_MethodGroup
    class GetService_MethodGroup:
        def __getitem__(self, t:typing.Type[GetService_1_T1]) -> GetService_1[GetService_1_T1]: ...

        GetService_1_T1 = typing.TypeVar('GetService_1_T1')
        class GetService_1(typing.Generic[GetService_1_T1]):
            GetService_1_T = ServicesProvider.GetService_MethodGroup.GetService_1_T1
            def __call__(self) -> GetService_1_T:...




class SimQuaternionExtensions(abc.ABC):
    @staticmethod
    def ToEulerAngles(q: SimQuaternion) -> SimVector3D: ...
    # Skipped CreateFromYawPitchRoll due to it being static, abstract and generic.

    CreateFromYawPitchRoll : CreateFromYawPitchRoll_MethodGroup
    class CreateFromYawPitchRoll_MethodGroup:
        @typing.overload
        def __call__(self, eulerAngles: SimVector3D) -> SimQuaternion:...
        @typing.overload
        def __call__(self, yaw: float, pitch: float, roll: float) -> SimQuaternion:...



class SystemTimerFactory(IDispatcherTimerFactory):
    def __init__(self) -> None: ...
    def Create(self) -> IDispatcherTimer: ...


class TaxonomyUtils(abc.ABC):
    @staticmethod
    def DuplicateLanguage(taxonomy: SimTaxonomy, source: CultureInfo, target: CultureInfo) -> bool: ...
    @staticmethod
    def GetLocalizedName(entry: SimTaxonomyEntryOrString, culture: CultureInfo) -> str: ...


class UnsyncedSynchronizationContext(ISynchronizeInvoke):
    def __init__(self) -> None: ...
    @property
    def InvokeRequired(self) -> bool: ...
    def BeginInvoke(self, method: Delegate, args: Array_1[typing.Any]) -> IAsyncResult: ...
    def EndInvoke(self, result: IAsyncResult) -> typing.Any: ...
    def Invoke(self, method: Delegate, args: Array_1[typing.Any]) -> typing.Any: ...




import typing, abc
from System import Guid, Array_1

class ISymbolDocumentWriter(typing.Protocol):
    @abc.abstractmethod
    def SetCheckSum(self, algorithmId: Guid, checkSum: Array_1[int]) -> None: ...
    @abc.abstractmethod
    def SetSource(self, source: Array_1[int]) -> None: ...




import abc
from SIMULTAN.Serializer.DXF import DXFSectionParserElement_1
from SIMULTAN.Data.SitePlanner import SitePlannerProject
from System.IO import FileInfo
from SIMULTAN.Projects import ProjectData

class SiteplannerDxfIO(abc.ABC):
    @classmethod
    @property
    def SiteplannerSectionElement(cls) -> DXFSectionParserElement_1[SitePlannerProject]: ...
    @staticmethod
    def CreateEmptySitePlannerProject(file: FileInfo, projectData: ProjectData) -> None: ...




import typing
from System import Exception, EventArgs, Attribute

class ExceptionDispatchInfo:
    @property
    def SourceException(self) -> Exception: ...
    @staticmethod
    def Capture(source: Exception) -> ExceptionDispatchInfo: ...
    @staticmethod
    def SetCurrentStackTrace(source: Exception) -> Exception: ...
    @staticmethod
    def SetRemoteStackTrace(source: Exception, stackTrace: str) -> Exception: ...
    # Skipped Throw due to it being static, abstract and generic.

    Throw : Throw_MethodGroup
    class Throw_MethodGroup:
        @typing.overload
        def __call__(self) -> None:...
        @typing.overload
        def __call__(self, source: Exception) -> None:...



class FirstChanceExceptionEventArgs(EventArgs):
    def __init__(self, exception: Exception) -> None: ...
    @property
    def Exception(self) -> Exception: ...


class HandleProcessCorruptedStateExceptionsAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...




import numpy as np

from numpy import array
from typing import Union, Optional
from weakref import WeakSet, WeakValueDictionary

from System import Array
from System.Collections.Generic import IEnumerable

from SIMULTAN.Data.Components import SimComponent, SimInstanceType

from SIMULTAN.Data.Geometry import (Layer, Vertex, Edge, PEdge, Face, Volume, EdgeLoop, OperationPermission,
                                    GeometryModelOperationPermissions, GeometryOperationPermissions,
                                    LayerOperationPermissions, GeometryModelData, GeometricOrientation,
                                    BaseEdgeContainer)

from SIMULTAN.Data.Geometry import FaceAlgorithms

from SIMULTAN.Data.Geometry import GeometryModel as NetGeometryModel

# from SIMULTAN.Utils import IDispatcherTimerFactory

from SIMULTAN.Data.SimMath import SimPoint3D

from SIMULTAN.Data.Geometry import FaceAlgorithms
from SIMULTAN.Data.Geometry import EdgeAlgorithms
from SIMULTAN.Data.Geometry import VolumeAlgorithms


# from ..utils import create_mapped_python_object, create_python_object
# from ..files import create_asset_from_file, FileInfo, create_asset_from_string
from .. import config
from .. import logger

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from ..simultan_object import SimultanObject
    from ..data_model import DataModel


class MetaMock(type):
    def __call__(cls, *args, **kwargs):
        wrapped_object = kwargs.get('wrapped_object', None)
        if wrapped_object is not None and hasattr(wrapped_object, 'Id'):
            obj = cls._cls_instances.get(wrapped_object.Id, None)
            if obj is not None:
                return obj

        obj = cls.__new__(cls)
        if wrapped_object is None:
            wrapped_object = obj.create_simultan_instance(*args, **kwargs)
        kwargs['wrapped_object'] = wrapped_object
        obj.__init__(*args, **kwargs)
        if hasattr(wrapped_object, 'Id'):
            cls._cls_instances[wrapped_object.Id] = obj
        return obj


class BaseGeometry(object, metaclass=MetaMock):

    associate_instance_type = getattr(SimInstanceType, 'None')

    def __init__(self, *args, **kwargs):
        self._wrapped_object: Union[Vertex, Edge, Face, Volume, EdgeLoop, Layer] = kwargs.get('wrapped_object', None)
        self._geometry_model: Optional[GeometryModel] = kwargs.get('geometry_model', None)
        self._object_mapper = kwargs.get('object_mapper', config.get_default_mapper())
        self._data_model = kwargs.get('data_model', None)

    @property
    def id(self):
        return self._wrapped_object.Id

    @property
    def components(self):
        return self.get_components()

    def get_components(self) -> list[Union[SimComponent, 'SimultanObject']]:
        if self._object_mapper is None:
            return list(self._geometry_model.Exchange.GetComponents(self._wrapped_object))
        else:
            return [self._object_mapper.create_python_object(x,
                                                             data_model=self._data_model) for x in list(
                self._geometry_model.Exchange.GetComponents(self._wrapped_object))]

    def associate(self, component: Union[SimComponent, 'SimultanObject']):
        wrapped_obj = component if isinstance(component, SimComponent) else component._wrapped_obj
        wrapped_obj.InstanceType = self.associate_instance_type
        self._geometry_model.Exchange.Associate(wrapped_obj, self._wrapped_object)

    def disassociate(self, component: Union[SimComponent, 'SimultanObject']):
        wrapped_obj = component if isinstance(component, SimComponent) else component._wrapped_obj
        self._geometry_model.Exchange.Disassociate(wrapped_obj, self._wrapped_object)


class classproperty(object):

    def __init__(self, getter):
        self.getter = getter

    def __get__(self, instance, owner):
        return self.getter(owner)


class SimultanLayer(BaseGeometry):

    _cls_instances = WeakValueDictionary()  # weak set with all created objects

    @classmethod
    def create_simultan_instance(cls, *args, **kwargs) -> 'SimultanLayer':
        """
        Create a new SimultanLayer instance and add to the fc_geometry model
        :keyword geometry_model: GeometryModel
        :keyword name: str
        :return: SimultanLayer
        """

        layer = Layer(kwargs.get('geometry_model').GeometryModelData,
                          kwargs.get('name', 'Layer'))
        geometry_model: GeometryModel = kwargs.get('geometry_model')
        geometry_model.add_layer(layer)
        return layer

    def __init__(self, *args, **kwargs):
        super(SimultanLayer, self).__init__(*args, **kwargs)

    @property
    def color(self):
        return self._wrapped_object.Color


class ExtendedBaseGeometry(BaseGeometry):

    def __init__(self, *args, **kwargs):
        super(ExtendedBaseGeometry, self).__init__(*args, **kwargs)
        self.layer = kwargs.get('layer', None)

    @property
    def name(self):
        return self._wrapped_object.Name

    @name.setter
    def name(self, value: str):
        self._wrapped_object.Name = value

    @property
    def layer(self):
        cls = self._object_mapper.registered_geometry_classes[Layer]
        return cls(wrapped_object=self._wrapped_object.Layer,
                   geometry_model=self._geometry_model,
                   object_mapper=self._object_mapper,
                   data_model=self._data_model)

    @layer.setter
    def layer(self, value: Union[SimultanLayer, Layer]):
        if value is None:
            return

        if isinstance(value, SimultanLayer):
            new_layer = value._wrapped_object
        else:
            new_layer = value

        if self._wrapped_object.Layer.Id != new_layer.Id:
            self._wrapped_object.Layer = new_layer

    def add_to_model(self):
        self._wrapped_object.AddToModel()

    def remove_from_model(self):
        self._wrapped_object.RemoveFromModel()


class SimultanVertex(ExtendedBaseGeometry):

    associate_instance_type = SimInstanceType.AttributesPoint
    _cls_instances = WeakValueDictionary()  # weak set with all created objects

    @classmethod
    def create_simultan_instance(cls, *args, **kwargs):
        k_pos: Optional[tuple[float, float, float]] = kwargs.get('position', None)
        if k_pos is not None:
            position = SimPoint3D(k_pos[0], k_pos[1], k_pos[2])
        else:
            position = SimPoint3D(kwargs.get('x', 0), kwargs.get('y', 0), kwargs.get('z', 0))

        # layer
        layer = kwargs.get('layer', None)
        if layer is None:
            layer = kwargs.get('geometry_model').default_layer

        if isinstance(layer, SimultanLayer):
            layer = layer._wrapped_object

        wrapped_object = Vertex(layer, kwargs.get('name', 'vertex'), position)
        return wrapped_object

    def __init__(self, *args, **kwargs):
        super(SimultanVertex, self).__init__(*args, **kwargs)
        if self._wrapped_object is None:
            self.x = kwargs.get('x', 0)
            self.y = kwargs.get('y', 0)
            self.z = kwargs.get('z', 0)

    @property
    def x(self):
        return self._wrapped_object.Position.X

    @x.setter
    def x(self, value: float):
        self._wrapped_object.Position.X = value

    @property
    def y(self):
        return self._wrapped_object.Position.Y

    @y.setter
    def y(self, value: float):
        self._wrapped_object.Position.Y = value

    @property
    def z(self):
        return self._wrapped_object.Position.Z

    @z.setter
    def z(self, value: float):
        self._wrapped_object.Position.Z = value

    @property
    def position(self):
        return (self._wrapped_object.Position.X,
                self._wrapped_object.Position.Y,
                self._wrapped_object.Position.Z)

    @position.setter
    def position(self, value: tuple[float, float, float]):
        self._wrapped_object.Position.X = value[0]
        self._wrapped_object.Position.Y = value[1]
        self._wrapped_object.Position.Z = value[2]

    def __repr__(self):
        return f"SimultanVertex {self.id} ({self.x}, {self.y}, {self.z})"


class SimultanEdge(ExtendedBaseGeometry):

    associate_instance_type = SimInstanceType.AttributesEdge
    _cls_instances = WeakValueDictionary()  # weak set with all created objects

    @classmethod
    def create_simultan_instance(cls, *args, **kwargs):
        # layer
        layer = kwargs.get('layer', None)
        if layer is None:
            layer = kwargs.get('geometry_model').default_layer

        if isinstance(layer, SimultanLayer):
            layer = layer._wrapped_object

        vertices = [x if isinstance(x, Vertex) else x._wrapped_object for x in kwargs.get('vertices', [])]

        wrapped_object = Edge(layer,
                              kwargs.get('name', 'vertex'),
                              IEnumerable[Vertex](Array[Vertex](vertices)))
        return wrapped_object

    def __init__(self, *args, **kwargs):
        super(SimultanEdge, self).__init__(*args, **kwargs)

    @property
    def length(self):
        return EdgeAlgorithms.Length(self._wrapped_object)

    @property
    def vertices(self):
        cls = self._object_mapper.registered_geometry_classes[Vertex]
        return [cls(wrapped_object=x,
                    geometry_model=self._geometry_model,
                    object_mapper=self._object_mapper,
                    data_model=self._data_model) for x in self._wrapped_object.Vertices]

    @property
    def vertex_0(self):
        cls = self._object_mapper.registered_geometry_classes[Vertex]
        return cls(wrapped_object=self._wrapped_object.Vertices[0],
                   geometry_model=self._geometry_model,
                   object_mapper=self._object_mapper,
                   data_model=self._data_model)

    @property
    def vertex_1(self):
        cls = self._object_mapper.registered_geometry_classes[Vertex]
        return cls(wrapped_object=self._wrapped_object.Vertices[1],
                   geometry_model=self._geometry_model,
                   object_mapper=self._object_mapper,
                   data_model=self._data_model)

    def __repr__(self):
        return f"SimultanEdge {self.id} ({self.vertex_0.id}, {self.vertex_1.id})"


class SimultanPEdge(ExtendedBaseGeometry):

    _cls_instances = WeakValueDictionary()  # weak set with all created objects

    @classmethod
    def create_simultan_instance(cls, *args, **kwargs):

        layer = kwargs.get('layer', None)
        if layer is None:
            layer = kwargs.get('geometry_model').default_layer
        if isinstance(layer, SimultanLayer):
            layer = layer._wrapped_object

        edge = kwargs.get('edge', None)
        edge = edge._wrapped_object if isinstance(edge, SimultanEdge) else edge

        free_id = kwargs.get('geometry_model').get_free_id()

        wrapped_object = PEdge(edge,
                               kwargs.get('geometric_orientation', GeometricOrientation.Undefined),
                               kwargs.get('base_edge_container', BaseEdgeContainer(free_id, layer))
                               )
        return wrapped_object

    @property
    def id(self):
        return self._wrapped_object.GetHashCode()

    def __init__(self, *args, **kwargs):
        super(SimultanPEdge, self).__init__(*args, **kwargs)

    @property
    def edges(self):
        return [SimultanEdge(wrapped_object=x,
                             geometry_model=self._geometry_model,
                             object_mapper=self._object_mapper,
                             data_model=self._data_model) for x in self._wrapped_object.Edges]


class SimultanEdgeLoop(ExtendedBaseGeometry):

    _cls_instances = WeakValueDictionary()  # weak set with all created objects

    @classmethod
    def create_simultan_instance(cls, *args, **kwargs):
        # layer
        layer = kwargs.get('layer', None)
        if layer is None:
            layer = kwargs.get('geometry_model').default_layer

        if isinstance(layer, SimultanLayer):
            layer = layer._wrapped_object

        edges = [x if isinstance(x, Edge) else x._wrapped_object for x in kwargs.get('edges', [])]

        wrapped_object = EdgeLoop(layer,
                                  kwargs.get('name', 'EdgeLoop'),
                                  IEnumerable[Edge](Array[Edge](edges))
                                  )
        return wrapped_object

    def __init__(self, *args, **kwargs):
        super(SimultanEdgeLoop, self).__init__(*args, **kwargs)

    @property
    def edges(self):
        cls = self._object_mapper.registered_geometry_classes[Edge]
        return [cls(wrapped_object=x.Edge,
                    geometry_model=self._geometry_model,
                    object_mapper=self._object_mapper,
                    data_model=self._data_model) for x in self._wrapped_object.Edges]

    @property
    def vertices(self) -> list[SimultanVertex]:
        vertices = []

        for edge in self.edges:
            vertices.append(edge.vertices[0])

        return vertices

    def __repr__(self):
        return f"SimultanEdgeLoop {self.id} ({[x.id for x in self.edges]})"


class SimultanFace(ExtendedBaseGeometry):

    associate_instance_type = SimInstanceType.AttributesFace
    _cls_instances = WeakValueDictionary()  # weak set with all created objects

    @classmethod
    def create_simultan_instance(cls, *args, **kwargs):
        layer = kwargs.get('layer', None)
        if layer is None:
            layer = kwargs.get('geometry_model').default_layer
        if isinstance(layer, SimultanLayer):
            layer = layer._wrapped_object

        edge_loop = kwargs.get('edge_loop', kwargs.get('boundary', None))
        edge_loop = edge_loop if isinstance(edge_loop, EdgeLoop) else edge_loop._wrapped_object

        holes = [x if isinstance(x, EdgeLoop) else x._wrapped_object for x in kwargs.get('holes', [])]
        if len(holes) > 0:
            holes = IEnumerable[EdgeLoop](Array[EdgeLoop](holes))
        else:
            holes = None

        wrapped_object = Face(layer,
                              kwargs.get('name', 'Face'),
                              edge_loop,
                              kwargs.get('geometric_orientation', GeometricOrientation.Forward),
                              holes)

        return wrapped_object

    def __init__(self, *args, **kwargs):
        super(SimultanFace, self).__init__(*args, **kwargs)

    @property
    def area(self) -> tuple[np.ndarray, np.ndarray]:
        return FaceAlgorithms.Area(self._wrapped_object)

    @property
    def boundary(self):
        cls = self._object_mapper.registered_geometry_classes[EdgeLoop]
        return cls(wrapped_object=self._wrapped_object.Boundary,
                   geometry_model=self._geometry_model,
                   object_mapper=self._object_mapper,
                   data_model=self._data_model)

    @property
    def holes(self) -> list[SimultanEdgeLoop]:
        cls = self._object_mapper.registered_geometry_classes[EdgeLoop]
        return [cls(wrapped_object=x,
                    geometry_model=self._geometry_model,
                    object_mapper=self._object_mapper,
                    data_model=self._data_model) for x in self._wrapped_object.Holes]

    @property
    def orientation(self):
        return self._wrapped_object.Orientation

    @property
    def normal(self):
        return array([self._wrapped_object.Normal.X,
                      self._wrapped_object.Normal.Y,
                      self._wrapped_object.Normal.Z])

    @property
    def stl(self):
        return self.create_stl_str()

    @property
    def vertices(self) -> list[SimultanVertex]:
        vertices = self.boundary_vertices
        for hole in self.holes:
            vertices.append(hole.vertices[0])
        return vertices

    @property
    def boundary_vertices(self) -> list[SimultanVertex]:
        vertices = []
        for edge in self.boundary.edges:
            vertices.append(edge.vertices[0])
        return vertices

    def triangulate(self):
        triangulation =FaceAlgorithms.Triangulate(self._wrapped_object, GeometricOrientation.Forward)
        vertices = np.array([(x.X, x.Y, x.Z) for x in triangulation.Item1])
        triangles = np.array(triangulation.Item3).reshape(int(triangulation.Item3.Count / 3), 3)

        return vertices, triangles

    def get_orientation_to_volume(self, volume: 'SimultanVolume') -> Optional[GeometricOrientation]:

        p_face = next((x for x in volume._wrapped_object.Faces if x.Face.Id is self._wrapped_object.Id), None)

        if p_face is None or p_face.Orientation == GeometricOrientation.Undefined:
            return None

        mul_1 = 1 if self.orientation == GeometricOrientation.Forward else -1
        mul_2 = 1 if p_face.Orientation == GeometricOrientation.Forward else -1

        return GeometricOrientation.Forward if mul_1 * mul_2 == 1 else GeometricOrientation.Backward

    def __repr__(self):
        return f"SimultanFace {self.id} ({self.boundary.id}, {[x.id for x in self.holes]})"


class SimultanVolume(ExtendedBaseGeometry):

    associate_instance_type = SimInstanceType.Entity3D
    _cls_instances = WeakValueDictionary()  # weak set with all created objects

    @classmethod
    def create_simultan_instance(cls, *args, **kwargs):
        layer = kwargs.get('layer', None)
        if layer is None:
            layer = kwargs.get('geometry_model').default_layer

        if isinstance(layer, SimultanLayer):
            layer = layer._wrapped_object

        faces = IEnumerable[Face](
            Array[Face]([x if isinstance(x, Face) else x._wrapped_object for x in kwargs.get('faces', [])]))

        wrapped_object = Volume(layer,
                                kwargs.get('name', 'Face'),
                                faces,
                                )

        return wrapped_object

    def __init__(self, *args, **kwargs):
        super(SimultanVolume, self).__init__(*args, **kwargs)

    @property
    def faces(self):

        face_cls = self._object_mapper.registered_geometry_classes[Face]
        return [face_cls(wrapped_object=x.Face,
                         geometry_model=self._geometry_model,
                         object_mapper=self._object_mapper,
                         data_model=self._data_model) for x in self._wrapped_object.Faces]

    @property
    def vertices(self):
        vertices = set()

        for face in self.faces:
            vertices.update(face.vertices)

        return vertices

    @property
    def area_brutto_netto(self):
        return VolumeAlgorithms.AreaBruttoNetto(self._wrapped_object)

    @property
    def volume(self):
        return VolumeAlgorithms.Volume(self._wrapped_object)

    def __repr__(self):
        return f"SimultanVolume {self.id} ({[x.id for x in self.faces]})"


class GeometryModel(object, metaclass=MetaMock):

    @classmethod
    def create_simultan_instance(cls, *args, **kwargs):

        data_model: DataModel = kwargs.get('data_model', config.get_default_data_model())
        name: str = kwargs.get('name', 'GeometryModel')

        new_geo_model, resource = data_model.create_new_geometry_model(name=name)
        data_model.project_data_manager.GeometryModels.AddGeometryModel(new_geo_model)
        data_model.models_dict[resource.Key] = new_geo_model
        data_model.save()

        return new_geo_model

    def __init__(self, *args, **kwargs):
        self._wrapped_object: NetGeometryModel = kwargs.get('wrapped_object', None)
        self._object_mapper = kwargs.get('object_mapper', config.get_default_mapper())
        self._data_model = kwargs.get('data_model', config.get_default_data_model())

    @property
    def name(self):
        return self._wrapped_object.Name

    @name.setter
    def name(self, value: str):
        self._wrapped_object.Name = value

    @property
    def key(self):
        if self.resource_file_entry is not None:
            return self.resource_file_entry.Key

    @property
    def layers(self):
        cls = self._object_mapper.registered_geometry_classes[Layer]
        return [cls(wrapped_object=x,
                    geometry_model=self,
                    object_mapper=self._object_mapper,
                    data_model=self._data_model) for x in list(self._wrapped_object.Geometry.Layers)]

    @property
    def default_layer(self):
        if self.layers:
            return self.layers[0]
        else:
            cls = self._object_mapper.registered_geometry_classes[Layer]
            cls(geometry_model=self,
                data_model=self._data_model,
                name='Default Layer')
        return self.layers[0]

    @property
    def vertices(self):
        cls = self._object_mapper.registered_geometry_classes[Vertex]
        return [cls(wrapped_object=x,
                    geometry_model=self,
                    object_mapper=self._object_mapper,
                    data_model=self._data_model) for x in self._wrapped_object.Geometry.Vertices]

    @property
    def edges(self):
        cls = self._object_mapper.registered_geometry_classes[Edge]
        return [cls(wrapped_object=x,
                    geometry_model=self,
                    object_mapper=self._object_mapper,
                    data_model=self._data_model) for x in self._wrapped_object.Geometry.Edges]

    @property
    def edge_loops(self):
        cls = self._object_mapper.registered_geometry_classes[EdgeLoop]
        return [cls(wrapped_object=x,
                    geometry_model=self,
                    object_mapper=self._object_mapper,
                    data_model=self._data_model) for x in self._wrapped_object.Geometry.EdgeLoops]

    @property
    def faces(self):
        cls = self._object_mapper.registered_geometry_classes[Face]
        return [cls(wrapped_object=x,
                    geometry_model=self,
                    object_mapper=self._object_mapper,
                    data_model=self._data_model) for x in self._wrapped_object.Geometry.Faces]

    @property
    def volumes(self):
        cls = self._object_mapper.registered_geometry_classes[Volume]
        return [cls(wrapped_object=x,
                    geometry_model=self,
                    object_mapper=self._object_mapper,
                    data_model=self._data_model) for x in self._wrapped_object.Geometry.Volumes]

    @property
    def resource_file_entry(self):
        return self._wrapped_object.File

    @property
    def geo_references(self):
        return self._wrapped_object.Geometry.GeoReferences

    @property
    def Exchange(self):
        return self._wrapped_object.Exchange

    @property
    def GeometryModelData(self):
        return self._wrapped_object.Geometry

    def add_layer(self, layer: Union[Layer, SimultanLayer]):
        if isinstance(layer, SimultanLayer):
            layer = layer._wrapped_object
        self._wrapped_object.Geometry.Layers.Add(layer)

    def get_geometry_by_id(self, geo_id):
        return self._wrapped_object.Geometry.GeometryFromId(geo_id)

    def get_layer_by_id(self, geo_id):
        return self._wrapped_object.Geometry.LayerFromId(geo_id)

    def get_free_id(self):
        return self._wrapped_object.Geometry.GetFreeId(True)



import typing, clr, abc
from System import Array_1, IDisposable, Memory_1, Span_1, MulticastDelegate, IAsyncResult, ReadOnlySpan_1, AsyncCallback, IEquatable_1
from System.Runtime.InteropServices import GCHandle
from System.Reflection import MethodInfo

class ArrayPool_GenericClasses(abc.ABCMeta):
    Generic_ArrayPool_GenericClasses_ArrayPool_1_T = typing.TypeVar('Generic_ArrayPool_GenericClasses_ArrayPool_1_T')
    def __getitem__(self, types : typing.Type[Generic_ArrayPool_GenericClasses_ArrayPool_1_T]) -> typing.Type[ArrayPool_1[Generic_ArrayPool_GenericClasses_ArrayPool_1_T]]: ...

ArrayPool : ArrayPool_GenericClasses

ArrayPool_1_T = typing.TypeVar('ArrayPool_1_T')
class ArrayPool_1(typing.Generic[ArrayPool_1_T], abc.ABC):
    @classmethod
    @property
    def Shared(cls) -> ArrayPool_1[ArrayPool_1_T]: ...
    @abc.abstractmethod
    def Rent(self, minimumLength: int) -> Array_1[ArrayPool_1_T]: ...
    @abc.abstractmethod
    def Return(self, array: Array_1[ArrayPool_1_T], clearArray: bool = ...) -> None: ...
    # Skipped Create due to it being static, abstract and generic.

    Create : Create_MethodGroup[ArrayPool_1_T]
    Create_MethodGroup_ArrayPool_1_T = typing.TypeVar('Create_MethodGroup_ArrayPool_1_T')
    class Create_MethodGroup(typing.Generic[Create_MethodGroup_ArrayPool_1_T]):
        Create_MethodGroup_ArrayPool_1_T = ArrayPool_1.Create_MethodGroup_ArrayPool_1_T
        @typing.overload
        def __call__(self) -> ArrayPool_1[Create_MethodGroup_ArrayPool_1_T]:...
        @typing.overload
        def __call__(self, maxArrayLength: int, maxArraysPerBucket: int) -> ArrayPool_1[Create_MethodGroup_ArrayPool_1_T]:...



class IMemoryOwner_GenericClasses(abc.ABCMeta):
    Generic_IMemoryOwner_GenericClasses_IMemoryOwner_1_T = typing.TypeVar('Generic_IMemoryOwner_GenericClasses_IMemoryOwner_1_T')
    def __getitem__(self, types : typing.Type[Generic_IMemoryOwner_GenericClasses_IMemoryOwner_1_T]) -> typing.Type[IMemoryOwner_1[Generic_IMemoryOwner_GenericClasses_IMemoryOwner_1_T]]: ...

IMemoryOwner : IMemoryOwner_GenericClasses

IMemoryOwner_1_T = typing.TypeVar('IMemoryOwner_1_T')
class IMemoryOwner_1(typing.Generic[IMemoryOwner_1_T], IDisposable, typing.Protocol):
    @property
    def Memory(self) -> Memory_1[IMemoryOwner_1_T]: ...


class IPinnable(typing.Protocol):
    @abc.abstractmethod
    def Pin(self, elementIndex: int) -> MemoryHandle: ...
    @abc.abstractmethod
    def Unpin(self) -> None: ...


class MemoryHandle(IDisposable):
    def __init__(self, pointer: clr.Reference[None], handle: GCHandle = ..., pinnable: IPinnable = ...) -> None: ...
    @property
    def Pointer(self) -> clr.Reference[None]: ...
    def Dispose(self) -> None: ...


class MemoryManager_GenericClasses(abc.ABCMeta):
    Generic_MemoryManager_GenericClasses_MemoryManager_1_T = typing.TypeVar('Generic_MemoryManager_GenericClasses_MemoryManager_1_T')
    def __getitem__(self, types : typing.Type[Generic_MemoryManager_GenericClasses_MemoryManager_1_T]) -> typing.Type[MemoryManager_1[Generic_MemoryManager_GenericClasses_MemoryManager_1_T]]: ...

MemoryManager : MemoryManager_GenericClasses

MemoryManager_1_T = typing.TypeVar('MemoryManager_1_T')
class MemoryManager_1(typing.Generic[MemoryManager_1_T], IMemoryOwner_1[MemoryManager_1_T], IPinnable, abc.ABC):
    @property
    def Memory(self) -> Memory_1[MemoryManager_1_T]: ...
    @abc.abstractmethod
    def GetSpan(self) -> Span_1[MemoryManager_1_T]: ...
    @abc.abstractmethod
    def Pin(self, elementIndex: int = ...) -> MemoryHandle: ...
    @abc.abstractmethod
    def Unpin(self) -> None: ...


class OperationStatus(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Done : OperationStatus # 0
    DestinationTooSmall : OperationStatus # 1
    NeedMoreData : OperationStatus # 2
    InvalidData : OperationStatus # 3


class ReadOnlySpanAction_GenericClasses(abc.ABCMeta):
    Generic_ReadOnlySpanAction_GenericClasses_ReadOnlySpanAction_2_T = typing.TypeVar('Generic_ReadOnlySpanAction_GenericClasses_ReadOnlySpanAction_2_T')
    Generic_ReadOnlySpanAction_GenericClasses_ReadOnlySpanAction_2_TArg = typing.TypeVar('Generic_ReadOnlySpanAction_GenericClasses_ReadOnlySpanAction_2_TArg')
    def __getitem__(self, types : typing.Tuple[typing.Type[Generic_ReadOnlySpanAction_GenericClasses_ReadOnlySpanAction_2_T], typing.Type[Generic_ReadOnlySpanAction_GenericClasses_ReadOnlySpanAction_2_TArg]]) -> typing.Type[ReadOnlySpanAction_2[Generic_ReadOnlySpanAction_GenericClasses_ReadOnlySpanAction_2_T, Generic_ReadOnlySpanAction_GenericClasses_ReadOnlySpanAction_2_TArg]]: ...

ReadOnlySpanAction : ReadOnlySpanAction_GenericClasses

ReadOnlySpanAction_2_T = typing.TypeVar('ReadOnlySpanAction_2_T')
ReadOnlySpanAction_2_TArg = typing.TypeVar('ReadOnlySpanAction_2_TArg', contravariant=True)
class ReadOnlySpanAction_2(typing.Generic[ReadOnlySpanAction_2_T, ReadOnlySpanAction_2_TArg], MulticastDelegate):
    def __init__(self, object: typing.Any, method: int) -> None: ...
    @property
    def Method(self) -> MethodInfo: ...
    @property
    def Target(self) -> typing.Any: ...
    def BeginInvoke(self, span: ReadOnlySpan_1[ReadOnlySpanAction_2_T], arg: ReadOnlySpanAction_2_TArg, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
    def EndInvoke(self, result: IAsyncResult) -> None: ...
    def Invoke(self, span: ReadOnlySpan_1[ReadOnlySpanAction_2_T], arg: ReadOnlySpanAction_2_TArg) -> None: ...


class SearchValues_GenericClasses(abc.ABCMeta):
    Generic_SearchValues_GenericClasses_SearchValues_1_T = typing.TypeVar('Generic_SearchValues_GenericClasses_SearchValues_1_T')
    def __getitem__(self, types : typing.Type[Generic_SearchValues_GenericClasses_SearchValues_1_T]) -> typing.Type[SearchValues_1[Generic_SearchValues_GenericClasses_SearchValues_1_T]]: ...

class SearchValues(SearchValues_0, metaclass =SearchValues_GenericClasses): ...

class SearchValues_0(abc.ABC):
    # Skipped Create due to it being static, abstract and generic.

    Create : Create_MethodGroup
    class Create_MethodGroup:
        @typing.overload
        def __call__(self, values: ReadOnlySpan_1[int]) -> SearchValues_1[int]:...
        @typing.overload
        def __call__(self, values: ReadOnlySpan_1[str]) -> SearchValues_1[str]:...



SearchValues_1_T = typing.TypeVar('SearchValues_1_T')
class SearchValues_1(typing.Generic[SearchValues_1_T]):
    def Contains(self, value: SearchValues_1_T) -> bool: ...


class SpanAction_GenericClasses(abc.ABCMeta):
    Generic_SpanAction_GenericClasses_SpanAction_2_T = typing.TypeVar('Generic_SpanAction_GenericClasses_SpanAction_2_T')
    Generic_SpanAction_GenericClasses_SpanAction_2_TArg = typing.TypeVar('Generic_SpanAction_GenericClasses_SpanAction_2_TArg')
    def __getitem__(self, types : typing.Tuple[typing.Type[Generic_SpanAction_GenericClasses_SpanAction_2_T], typing.Type[Generic_SpanAction_GenericClasses_SpanAction_2_TArg]]) -> typing.Type[SpanAction_2[Generic_SpanAction_GenericClasses_SpanAction_2_T, Generic_SpanAction_GenericClasses_SpanAction_2_TArg]]: ...

SpanAction : SpanAction_GenericClasses

SpanAction_2_T = typing.TypeVar('SpanAction_2_T')
SpanAction_2_TArg = typing.TypeVar('SpanAction_2_TArg', contravariant=True)
class SpanAction_2(typing.Generic[SpanAction_2_T, SpanAction_2_TArg], MulticastDelegate):
    def __init__(self, object: typing.Any, method: int) -> None: ...
    @property
    def Method(self) -> MethodInfo: ...
    @property
    def Target(self) -> typing.Any: ...
    def BeginInvoke(self, span: Span_1[SpanAction_2_T], arg: SpanAction_2_TArg, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
    def EndInvoke(self, result: IAsyncResult) -> None: ...
    def Invoke(self, span: Span_1[SpanAction_2_T], arg: SpanAction_2_TArg) -> None: ...


class StandardFormat(IEquatable_1[StandardFormat]):
    def __init__(self, symbol: str, precision: int = ...) -> None: ...
    MaxPrecision : int
    NoPrecision : int
    @property
    def HasPrecision(self) -> bool: ...
    @property
    def IsDefault(self) -> bool: ...
    @property
    def Precision(self) -> int: ...
    @property
    def Symbol(self) -> str: ...
    def GetHashCode(self) -> int: ...
    def __eq__(self, left: StandardFormat, right: StandardFormat) -> bool: ...
    # Operator not supported op_Implicit(symbol: Char)
    def __ne__(self, left: StandardFormat, right: StandardFormat) -> bool: ...
    def ToString(self) -> str: ...
    @staticmethod
    def TryParse(format: ReadOnlySpan_1[str], result: clr.Reference[StandardFormat]) -> bool: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, other: StandardFormat) -> bool:...
        @typing.overload
        def __call__(self, obj: typing.Any) -> bool:...

    # Skipped Parse due to it being static, abstract and generic.

    Parse : Parse_MethodGroup
    class Parse_MethodGroup:
        @typing.overload
        def __call__(self, format: ReadOnlySpan_1[str]) -> StandardFormat:...
        @typing.overload
        def __call__(self, format: str) -> StandardFormat:...





import typing, abc
from System import Attribute
from System.Security import IPermission

class CodeAccessSecurityAttribute(SecurityAttribute):
    @property
    def Action(self) -> SecurityAction: ...
    @Action.setter
    def Action(self, value: SecurityAction) -> SecurityAction: ...
    @property
    def TypeId(self) -> typing.Any: ...
    @property
    def Unrestricted(self) -> bool: ...
    @Unrestricted.setter
    def Unrestricted(self, value: bool) -> bool: ...


class PermissionState(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : PermissionState # 0
    Unrestricted : PermissionState # 1


class SecurityAction(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Demand : SecurityAction # 2
    Assert : SecurityAction # 3
    Deny : SecurityAction # 4
    PermitOnly : SecurityAction # 5
    LinkDemand : SecurityAction # 6
    InheritanceDemand : SecurityAction # 7
    RequestMinimum : SecurityAction # 8
    RequestOptional : SecurityAction # 9
    RequestRefuse : SecurityAction # 10


class SecurityAttribute(Attribute):
    @property
    def Action(self) -> SecurityAction: ...
    @Action.setter
    def Action(self, value: SecurityAction) -> SecurityAction: ...
    @property
    def TypeId(self) -> typing.Any: ...
    @property
    def Unrestricted(self) -> bool: ...
    @Unrestricted.setter
    def Unrestricted(self, value: bool) -> bool: ...
    @abc.abstractmethod
    def CreatePermission(self) -> IPermission: ...


class SecurityPermissionAttribute(CodeAccessSecurityAttribute):
    def __init__(self, action: SecurityAction) -> None: ...
    @property
    def Action(self) -> SecurityAction: ...
    @Action.setter
    def Action(self, value: SecurityAction) -> SecurityAction: ...
    @property
    def Assertion(self) -> bool: ...
    @Assertion.setter
    def Assertion(self, value: bool) -> bool: ...
    @property
    def BindingRedirects(self) -> bool: ...
    @BindingRedirects.setter
    def BindingRedirects(self, value: bool) -> bool: ...
    @property
    def ControlAppDomain(self) -> bool: ...
    @ControlAppDomain.setter
    def ControlAppDomain(self, value: bool) -> bool: ...
    @property
    def ControlDomainPolicy(self) -> bool: ...
    @ControlDomainPolicy.setter
    def ControlDomainPolicy(self, value: bool) -> bool: ...
    @property
    def ControlEvidence(self) -> bool: ...
    @ControlEvidence.setter
    def ControlEvidence(self, value: bool) -> bool: ...
    @property
    def ControlPolicy(self) -> bool: ...
    @ControlPolicy.setter
    def ControlPolicy(self, value: bool) -> bool: ...
    @property
    def ControlPrincipal(self) -> bool: ...
    @ControlPrincipal.setter
    def ControlPrincipal(self, value: bool) -> bool: ...
    @property
    def ControlThread(self) -> bool: ...
    @ControlThread.setter
    def ControlThread(self, value: bool) -> bool: ...
    @property
    def Execution(self) -> bool: ...
    @Execution.setter
    def Execution(self, value: bool) -> bool: ...
    @property
    def Flags(self) -> SecurityPermissionFlag: ...
    @Flags.setter
    def Flags(self, value: SecurityPermissionFlag) -> SecurityPermissionFlag: ...
    @property
    def Infrastructure(self) -> bool: ...
    @Infrastructure.setter
    def Infrastructure(self, value: bool) -> bool: ...
    @property
    def RemotingConfiguration(self) -> bool: ...
    @RemotingConfiguration.setter
    def RemotingConfiguration(self, value: bool) -> bool: ...
    @property
    def SerializationFormatter(self) -> bool: ...
    @SerializationFormatter.setter
    def SerializationFormatter(self, value: bool) -> bool: ...
    @property
    def SkipVerification(self) -> bool: ...
    @SkipVerification.setter
    def SkipVerification(self, value: bool) -> bool: ...
    @property
    def TypeId(self) -> typing.Any: ...
    @property
    def UnmanagedCode(self) -> bool: ...
    @UnmanagedCode.setter
    def UnmanagedCode(self, value: bool) -> bool: ...
    @property
    def Unrestricted(self) -> bool: ...
    @Unrestricted.setter
    def Unrestricted(self, value: bool) -> bool: ...
    def CreatePermission(self) -> IPermission: ...


class SecurityPermissionFlag(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    NoFlags : SecurityPermissionFlag # 0
    Assertion : SecurityPermissionFlag # 1
    UnmanagedCode : SecurityPermissionFlag # 2
    SkipVerification : SecurityPermissionFlag # 4
    Execution : SecurityPermissionFlag # 8
    ControlThread : SecurityPermissionFlag # 16
    ControlEvidence : SecurityPermissionFlag # 32
    ControlPolicy : SecurityPermissionFlag # 64
    SerializationFormatter : SecurityPermissionFlag # 128
    ControlDomainPolicy : SecurityPermissionFlag # 256
    ControlPrincipal : SecurityPermissionFlag # 512
    ControlAppDomain : SecurityPermissionFlag # 1024
    RemotingConfiguration : SecurityPermissionFlag # 2048
    Infrastructure : SecurityPermissionFlag # 4096
    BindingRedirects : SecurityPermissionFlag # 8192
    AllFlags : SecurityPermissionFlag # 16383




import typing, clr, abc
from SIMULTAN.Data import SimNamedObject_1, SimId, SimManagedCollection_1, IReferenceLocation
from System import Guid, Array_1, EventArgs, MulticastDelegate, IAsyncResult, AsyncCallback, DateTime, ValueTuple_2, ValueTuple_3
from System.Collections.Generic import ICollection_1, List_1, HashSet_1, IEnumerable_1, Dictionary_2, IDictionary_2, KeyValuePair_2, IEnumerator_1
from SIMULTAN.Utils import RowColumnRange, IntIndex3D, Range3D
from SIMULTAN.Data.SimMath import SimPoint4D, SimPoint3D, SimRect
from System.Collections.ObjectModel import ObservableCollection_1
from System.Reflection import MethodInfo
from System.ComponentModel import INotifyPropertyChanged
from SIMULTAN.Data.Components import SimBaseParameter, SimParameterValueSource, SimComponentCollection
from SIMULTAN.Projects import ProjectData
from System.Collections.Specialized import INotifyCollectionChanged
from SIMULTAN.Data.Users import SimUser

class SimAggregationFunction(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Sum : SimAggregationFunction # 0
    Average : SimAggregationFunction # 1
    Min : SimAggregationFunction # 2
    Max : SimAggregationFunction # 3
    Count : SimAggregationFunction # 4


class SimAggregationFunctionExtensions(abc.ABC):
    @staticmethod
    def FromStringRepresentation(representation: str) -> SimAggregationFunction: ...
    @staticmethod
    def ToStringRepresentation(aggregationFunction: SimAggregationFunction) -> str: ...


class SimMultiValue(SimNamedObject_1[SimMultiValueCollection]):
    @property
    def CanInterpolate(self) -> bool: ...
    @CanInterpolate.setter
    def CanInterpolate(self, value: bool) -> bool: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def Factory(self) -> SimMultiValueCollection: ...
    @Factory.setter
    def Factory(self, value: SimMultiValueCollection) -> SimMultiValueCollection: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def Id(self) -> SimId: ...
    @Id.setter
    def Id(self, value: SimId) -> SimId: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def MVType(self) -> SimMultiValueType: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def UnitX(self) -> str: ...
    @UnitX.setter
    def UnitX(self, value: str) -> str: ...
    @property
    def UnitY(self) -> str: ...
    @UnitY.setter
    def UnitY(self, value: str) -> str: ...
    @property
    def UnitZ(self) -> str: ...
    @UnitZ.setter
    def UnitZ(self, value: str) -> str: ...
    @abc.abstractmethod
    def Clone(self) -> SimMultiValue: ...
    # Skipped CreateNewPointer due to it being static, abstract and generic.

    CreateNewPointer : CreateNewPointer_MethodGroup
    class CreateNewPointer_MethodGroup:
        @typing.overload
        def __call__(self) -> SimMultiValueParameterSource:...
        @typing.overload
        def __call__(self, source: SimMultiValueParameterSource) -> SimMultiValueParameterSource:...



class SimMultiValueBigTable(SimMultiValue):
    @typing.overload
    def __init__(self, name: str, unitColumns: str, unitRows: str, columnHeaders: ICollection_1[SimMultiValueBigTableHeader], rowHeaders: ICollection_1[SimMultiValueBigTableHeader], values: List_1[List_1[typing.Any]], checkValueTypes: bool = ...) -> None: ...
    @typing.overload
    def __init__(self, name: str, unitColumns: str, unitRows: str, columnHeaders: ICollection_1[SimMultiValueBigTableHeader], rowHeaders: ICollection_1[SimMultiValueBigTableHeader], values: List_1[List_1[float]], checkValueTypes: bool = ...) -> None: ...
    @typing.overload
    def __init__(self, name: str, unitColumns: str, unitRows: str, columnHeaders: ICollection_1[SimMultiValueBigTableHeader], rowHeaders: ICollection_1[SimMultiValueBigTableHeader], values: Array_1[typing.Any], checkValueTypes: bool = ...) -> None: ...
    @typing.overload
    def __init__(self, name: str, unitColumns: str, unitRows: str, columnHeaders: ICollection_1[SimMultiValueBigTableHeader], rowHeaders: ICollection_1[SimMultiValueBigTableHeader], values: Array_1[float], checkValueTypes: bool = ...) -> None: ...
    @property
    def AdditionalInfo(self) -> str: ...
    @AdditionalInfo.setter
    def AdditionalInfo(self, value: str) -> str: ...
    @property
    def CanInterpolate(self) -> bool: ...
    @CanInterpolate.setter
    def CanInterpolate(self, value: bool) -> bool: ...
    @property
    def ColumnHeaders(self) -> SimMultiValueBigTable.HeaderCollection: ...
    @ColumnHeaders.setter
    def ColumnHeaders(self, value: SimMultiValueBigTable.HeaderCollection) -> SimMultiValueBigTable.HeaderCollection: ...
    @property
    def DefaultPointer(self) -> SimMultiValueParameterSource: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def Factory(self) -> SimMultiValueCollection: ...
    @Factory.setter
    def Factory(self, value: SimMultiValueCollection) -> SimMultiValueCollection: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def Id(self) -> SimId: ...
    @Id.setter
    def Id(self, value: SimId) -> SimId: ...
    @property
    def Item(self) -> typing.Any: ...
    @Item.setter
    def Item(self, value: typing.Any) -> typing.Any: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def MVType(self) -> SimMultiValueType: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def RowHeaders(self) -> SimMultiValueBigTable.HeaderCollection: ...
    @RowHeaders.setter
    def RowHeaders(self, value: SimMultiValueBigTable.HeaderCollection) -> SimMultiValueBigTable.HeaderCollection: ...
    @classmethod
    @property
    def SupportedValueTypes(cls) -> HashSet_1[typing.Type[typing.Any]]: ...
    @property
    def UnitX(self) -> str: ...
    @UnitX.setter
    def UnitX(self, value: str) -> str: ...
    @property
    def UnitY(self) -> str: ...
    @UnitY.setter
    def UnitY(self, value: str) -> str: ...
    @property
    def UnitZ(self) -> str: ...
    @UnitZ.setter
    def UnitZ(self, value: str) -> str: ...
    def Clone(self) -> SimMultiValue: ...
    def Count(self, dimension: int) -> int: ...
    def GetColumn(self, idx: int) -> IEnumerable_1[typing.Any]: ...
    def GetDoubleRange(self, range: RowColumnRange) -> Array_1[float]: ...
    def GetRow(self, idx: int) -> IEnumerable_1[typing.Any]: ...
    def Resize(self, rows: int, columns: int) -> None: ...
    # Skipped CreateNewPointer due to it being static, abstract and generic.

    CreateNewPointer : CreateNewPointer_MethodGroup
    class CreateNewPointer_MethodGroup:
        @typing.overload
        def __call__(self) -> SimMultiValueParameterSource:...
        @typing.overload
        def __call__(self, source: SimMultiValueParameterSource) -> SimMultiValueParameterSource:...

    # Skipped GetRange due to it being static, abstract and generic.

    GetRange : GetRange_MethodGroup
    class GetRange_MethodGroup:
        def __getitem__(self, t:typing.Type[GetRange_1_T1]) -> GetRange_1[GetRange_1_T1]: ...

        GetRange_1_T1 = typing.TypeVar('GetRange_1_T1')
        class GetRange_1(typing.Generic[GetRange_1_T1]):
            GetRange_1_T = SimMultiValueBigTable.GetRange_MethodGroup.GetRange_1_T1
            def __call__(self, _range_definition: SimPoint4D) -> List_1[List_1[GetRange_1_T]]:...

        @typing.overload
        def __call__(self, _range_definition: SimPoint4D) -> List_1[List_1[typing.Any]]:...
        @typing.overload
        def __call__(self, range: RowColumnRange) -> Array_1[typing.Any]:...

    # Skipped ReplaceData due to it being static, abstract and generic.

    ReplaceData : ReplaceData_MethodGroup
    class ReplaceData_MethodGroup:
        @typing.overload
        def __call__(self, source: SimMultiValueBigTable) -> None:...
        @typing.overload
        def __call__(self, columnHeaders: ICollection_1[SimMultiValueBigTableHeader], rowHeaders: ICollection_1[SimMultiValueBigTableHeader], values: List_1[List_1[float]]) -> None:...
        @typing.overload
        def __call__(self, columnHeaders: ICollection_1[SimMultiValueBigTableHeader], rowHeaders: ICollection_1[SimMultiValueBigTableHeader], values: List_1[List_1[typing.Any]], checkValueTypes: bool = ...) -> None:...


    class HeaderCollection(ObservableCollection_1[SimMultiValueBigTableHeader]):
        @property
        def Count(self) -> int: ...
        @property
        def Item(self) -> SimMultiValueBigTableHeader: ...
        @Item.setter
        def Item(self, value: SimMultiValueBigTableHeader) -> SimMultiValueBigTableHeader: ...


    class HeaderValueChangedEventArgs(EventArgs):
        def __init__(self, index: int, axis: SimMultiValueBigTableHeader.AxisEnum) -> None: ...
        @property
        def Axis(self) -> SimMultiValueBigTableHeader.AxisEnum: ...
        @property
        def Index(self) -> int: ...


    class ResizeDirection(typing.SupportsInt):
        @typing.overload
        def __init__(self, value : int) -> None: ...
        @typing.overload
        def __init__(self, value : int, force_if_true: bool) -> None: ...
        def __int__(self) -> int: ...
        
        # Values:
        Rows : SimMultiValueBigTable.ResizeDirection # 1
        Columns : SimMultiValueBigTable.ResizeDirection # 2
        Both : SimMultiValueBigTable.ResizeDirection # 3


    class ResizeEventArgs(EventArgs):
        def __init__(self, direction: SimMultiValueBigTable.ResizeDirection, rowStartIndex: int, columnStartIndex: int) -> None: ...
        @property
        def ColumnStartIndex(self) -> int: ...
        @property
        def ResizeDirection(self) -> SimMultiValueBigTable.ResizeDirection: ...
        @property
        def RowStartIndex(self) -> int: ...


    class ValueChangedEventArgs(EventArgs):
        def __init__(self, row: int, column: int) -> None: ...
        @property
        def Column(self) -> int: ...
        @Column.setter
        def Column(self, value: int) -> int: ...
        @property
        def Row(self) -> int: ...
        @Row.setter
        def Row(self, value: int) -> int: ...


    class ValueChangedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, args: SimMultiValueBigTable.ValueChangedEventArgs, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, args: SimMultiValueBigTable.ValueChangedEventArgs) -> None: ...



class SimMultiValueBigTableHeader(INotifyPropertyChanged):
    def __init__(self, name: str, unit: str) -> None: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def Unit(self) -> str: ...
    @Unit.setter
    def Unit(self, value: str) -> str: ...
    def Clone(self) -> SimMultiValueBigTableHeader: ...

    class AxisEnum(typing.SupportsInt):
        @typing.overload
        def __init__(self, value : int) -> None: ...
        @typing.overload
        def __init__(self, value : int, force_if_true: bool) -> None: ...
        def __int__(self) -> int: ...
        
        # Values:
        Rows : SimMultiValueBigTableHeader.AxisEnum # 0
        Columns : SimMultiValueBigTableHeader.AxisEnum # 1
        Undefined : SimMultiValueBigTableHeader.AxisEnum # -1



class SimMultiValueBigTableParameterSource(SimMultiValueParameterSource):
    def __init__(self, table: SimMultiValueBigTable, row: int, column: int) -> None: ...
    @property
    def Column(self) -> int: ...
    @Column.setter
    def Column(self, value: int) -> int: ...
    @property
    def Row(self) -> int: ...
    @Row.setter
    def Row(self, value: int) -> int: ...
    @property
    def Table(self) -> SimMultiValueBigTable: ...
    @property
    def TargetParameter(self) -> SimBaseParameter: ...
    @TargetParameter.setter
    def TargetParameter(self, value: SimBaseParameter) -> SimBaseParameter: ...
    @property
    def ValueField(self) -> SimMultiValue: ...
    def Clone(self) -> SimParameterValueSource: ...
    def GetValue(self) -> typing.Any: ...
    def IsSamePointer(self, other: SimMultiValueParameterSource) -> bool: ...
    def SetFromParameters(self, axisValueX: float, axisValueY: float, axisValueZ: float, gs: str) -> None: ...


class SimMultiValueCollection(SimManagedCollection_1[SimMultiValue]):
    def __init__(self, owner: ProjectData) -> None: ...
    @property
    def CalledFromLocation(self) -> IReferenceLocation: ...
    @property
    def Count(self) -> int: ...
    @property
    def HasChanges(self) -> bool: ...
    @property
    def Item(self) -> SimMultiValue: ...
    @Item.setter
    def Item(self, value: SimMultiValue) -> SimMultiValue: ...
    @property
    def LastChange(self) -> DateTime: ...
    @property
    def ProjectData(self) -> ProjectData: ...
    def EndLoading(self) -> None: ...
    def GetByID(self, _location: Guid, _id: int) -> SimMultiValue: ...
    def Merge(self, source: SimMultiValueCollection) -> Dictionary_2[int, int]: ...
    def RemoveUnused(self, components: SimComponentCollection, _excluded_from_removal: IEnumerable_1[SimMultiValue]) -> None: ...
    def StartLoading(self) -> None: ...


class SimMultiValueField3D(SimMultiValue):
    @typing.overload
    def __init__(self, id: int, name: str, xaxis: IEnumerable_1[float], unitX: str, yaxis: IEnumerable_1[float], unitY: str, zaxis: IEnumerable_1[float], unitZ: str, data: IDictionary_2[SimPoint3D, float], canInterpolate: bool) -> None: ...
    @typing.overload
    def __init__(self, name: str, xaxis: IEnumerable_1[float], unitX: str, yaxis: IEnumerable_1[float], unitY: str, zaxis: IEnumerable_1[float], unitZ: str, data: IEnumerable_1[float], canInterpolate: bool) -> None: ...
    @property
    def CanInterpolate(self) -> bool: ...
    @CanInterpolate.setter
    def CanInterpolate(self, value: bool) -> bool: ...
    @property
    def DefaultPointer(self) -> SimMultiValueParameterSource: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def Factory(self) -> SimMultiValueCollection: ...
    @Factory.setter
    def Factory(self, value: SimMultiValueCollection) -> SimMultiValueCollection: ...
    @property
    def Field(self) -> IEnumerable_1[KeyValuePair_2[IntIndex3D, float]]: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def Id(self) -> SimId: ...
    @Id.setter
    def Id(self, value: SimId) -> SimId: ...
    @property
    def Item(self) -> float: ...
    @Item.setter
    def Item(self, value: float) -> float: ...
    @property
    def Item(self) -> float: ...
    @Item.setter
    def Item(self, value: float) -> float: ...
    @property
    def Length(self) -> int: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def MVType(self) -> SimMultiValueType: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def UnitX(self) -> str: ...
    @UnitX.setter
    def UnitX(self, value: str) -> str: ...
    @property
    def UnitY(self) -> str: ...
    @UnitY.setter
    def UnitY(self, value: str) -> str: ...
    @property
    def UnitZ(self) -> str: ...
    @UnitZ.setter
    def UnitZ(self, value: str) -> str: ...
    @property
    def XAxis(self) -> SimMultiValueField3D.AxisCollection: ...
    @property
    def YAxis(self) -> SimMultiValueField3D.AxisCollection: ...
    @property
    def ZAxis(self) -> SimMultiValueField3D.AxisCollection: ...
    def AxisPositionFromValue(self, axis: SimMultiValueField3D.Axis, value: float) -> float: ...
    def Clone(self) -> SimMultiValue: ...
    def GetAxis(self, axis: SimMultiValueField3D.Axis) -> ObservableCollection_1[float]: ...
    def GetValue(self, position: SimPoint3D) -> float: ...
    # Skipped Count due to it being static, abstract and generic.

    Count : Count_MethodGroup
    class Count_MethodGroup:
        @typing.overload
        def __call__(self, dimension: int) -> int:...
        @typing.overload
        def __call__(self, axis: SimMultiValueField3D.Axis) -> int:...

    # Skipped CreateNewPointer due to it being static, abstract and generic.

    CreateNewPointer : CreateNewPointer_MethodGroup
    class CreateNewPointer_MethodGroup:
        @typing.overload
        def __call__(self) -> SimMultiValueParameterSource:...
        @typing.overload
        def __call__(self, source: SimMultiValueParameterSource) -> SimMultiValueParameterSource:...

    # Skipped ValueFromAxisPosition due to it being static, abstract and generic.

    ValueFromAxisPosition : ValueFromAxisPosition_MethodGroup
    class ValueFromAxisPosition_MethodGroup:
        @typing.overload
        def __call__(self, axis: SimMultiValueField3D.Axis, axisPosition: float) -> float:...
        @typing.overload
        def __call__(self, axis: SimMultiValueField3D.Axis, axisPosition: float, isOutside: clr.Reference[bool]) -> float:...


    class Axis(typing.SupportsInt):
        @typing.overload
        def __init__(self, value : int) -> None: ...
        @typing.overload
        def __init__(self, value : int, force_if_true: bool) -> None: ...
        def __int__(self) -> int: ...
        
        # Values:
        X : SimMultiValueField3D.Axis # 0
        Y : SimMultiValueField3D.Axis # 1
        Z : SimMultiValueField3D.Axis # 2


    class AxisCollection(ObservableCollection_1[float]):
        @property
        def Count(self) -> int: ...
        @property
        def Item(self) -> float: ...
        @Item.setter
        def Item(self, value: float) -> float: ...


    class ValueChangedEventArgs(EventArgs):
        def __init__(self, range: Range3D) -> None: ...
        @property
        def Range(self) -> Range3D: ...
        @Range.setter
        def Range(self, value: Range3D) -> Range3D: ...


    class ValueChangedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, args: SimMultiValueField3D.ValueChangedEventArgs, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, args: SimMultiValueField3D.ValueChangedEventArgs) -> None: ...



class SimMultiValueField3DParameterSource(SimMultiValueParameterSource):
    def __init__(self, field3D: SimMultiValueField3D, axisValueX: float, axisValueY: float, axisValueZ: float) -> None: ...
    @property
    def AxisValueX(self) -> float: ...
    @property
    def AxisValueY(self) -> float: ...
    @property
    def AxisValueZ(self) -> float: ...
    @property
    def Field(self) -> SimMultiValueField3D: ...
    @property
    def TargetParameter(self) -> SimBaseParameter: ...
    @TargetParameter.setter
    def TargetParameter(self, value: SimBaseParameter) -> SimBaseParameter: ...
    @property
    def ValueField(self) -> SimMultiValue: ...
    def Clone(self) -> SimParameterValueSource: ...
    def GetValue(self) -> typing.Any: ...
    def IsSamePointer(self, other: SimMultiValueParameterSource) -> bool: ...
    def SetFromParameters(self, axisValueX: float, axisValueY: float, axisValueZ: float, gs: str) -> None: ...


class SimMultiValueFunction(SimMultiValue):
    @typing.overload
    def __init__(self, id: int, name: str, unitX: str, unitY: str, unitZ: str, bounds: SimRect, zaxis: IEnumerable_1[float], graphs: IEnumerable_1[SimMultiValueFunctionGraph]) -> None: ...
    @typing.overload
    def __init__(self, name: str, unitX: str, unitY: str, unitZ: str, bounds: SimRect, zaxis: IEnumerable_1[float], graphs: IEnumerable_1[SimMultiValueFunctionGraph]) -> None: ...
    @property
    def CanInterpolate(self) -> bool: ...
    @CanInterpolate.setter
    def CanInterpolate(self, value: bool) -> bool: ...
    @property
    def DefaultPointer(self) -> SimMultiValueParameterSource: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def Factory(self) -> SimMultiValueCollection: ...
    @Factory.setter
    def Factory(self, value: SimMultiValueCollection) -> SimMultiValueCollection: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def Graphs(self) -> SimMultiValueFunction.GraphsCollection: ...
    @property
    def Id(self) -> SimId: ...
    @Id.setter
    def Id(self, value: SimId) -> SimId: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def MVType(self) -> SimMultiValueType: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def Range(self) -> Range3D: ...
    @Range.setter
    def Range(self, value: Range3D) -> Range3D: ...
    @property
    def UnitX(self) -> str: ...
    @UnitX.setter
    def UnitX(self, value: str) -> str: ...
    @property
    def UnitY(self) -> str: ...
    @UnitY.setter
    def UnitY(self, value: str) -> str: ...
    @property
    def UnitZ(self) -> str: ...
    @UnitZ.setter
    def UnitZ(self, value: str) -> str: ...
    @property
    def ZAxis(self) -> SimMultiValueFunction.ZAxisCollection: ...
    def Clone(self) -> SimMultiValue: ...
    # Skipped CreateNewPointer due to it being static, abstract and generic.

    CreateNewPointer : CreateNewPointer_MethodGroup
    class CreateNewPointer_MethodGroup:
        @typing.overload
        def __call__(self) -> SimMultiValueParameterSource:...
        @typing.overload
        def __call__(self, source: SimMultiValueParameterSource) -> SimMultiValueParameterSource:...

    # Skipped GetValue due to it being static, abstract and generic.

    GetValue : GetValue_MethodGroup
    class GetValue_MethodGroup:
        @typing.overload
        def __call__(self, position: SimPoint3D) -> float:...
        @typing.overload
        def __call__(self, position: SimPoint3D, tolerance: float) -> float:...
        @typing.overload
        def __call__(self, position: SimPoint3D, tolerance: float, isValid: clr.Reference[bool], closestPoint: clr.Reference[SimPoint3D], closestGraph: clr.Reference[SimMultiValueFunctionGraph]) -> float:...

    # Skipped Sample due to it being static, abstract and generic.

    Sample : Sample_MethodGroup
    class Sample_MethodGroup:
        @typing.overload
        def __call__(self, start: float, end: float, stepSize: float) -> ValueTuple_2[List_1[List_1[float]], List_1[str]]:...
        @typing.overload
        def __call__(self, start: float, end: float, stepSize: float, zIndex: int) -> ValueTuple_2[List_1[List_1[float]], List_1[str]]:...


    class GraphsCollection(ObservableCollection_1[SimMultiValueFunctionGraph]):
        def __init__(self, owner: SimMultiValueFunction, graphs: IEnumerable_1[SimMultiValueFunctionGraph]) -> None: ...
        @property
        def Count(self) -> int: ...
        @property
        def Item(self) -> SimMultiValueFunctionGraph: ...
        @Item.setter
        def Item(self, value: SimMultiValueFunctionGraph) -> SimMultiValueFunctionGraph: ...


    class ZAxisCollection(ObservableCollection_1[float]):
        def __init__(self, owner: SimMultiValueFunction, initialValues: IEnumerable_1[float]) -> None: ...
        @property
        def Count(self) -> int: ...
        @property
        def Item(self) -> float: ...
        @Item.setter
        def Item(self, value: float) -> float: ...



class SimMultiValueFunctionGraph(INotifyPropertyChanged):
    def __init__(self, name: str, points: IEnumerable_1[SimPoint3D]) -> None: ...
    @property
    def Function(self) -> SimMultiValueFunction: ...
    @Function.setter
    def Function(self, value: SimMultiValueFunction) -> SimMultiValueFunction: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def Points(self) -> SimMultiValueFunctionPointList: ...
    def Clone(self) -> SimMultiValueFunctionGraph: ...
    def ClosestPoint(self, position: SimPoint3D) -> ValueTuple_3[SimPoint3D, float, float]: ...
    def GetValueFromX(self, x: float, previouseY: float = ...) -> float: ...
    def Sample(self, start: float, end: float, stepSize: float) -> List_1[float]: ...


class SimMultiValueFunctionParameterSource(SimMultiValueParameterSource):
    def __init__(self, function: SimMultiValueFunction, graphName: str, axisValueX: float, axisValueY: float) -> None: ...
    @property
    def AxisValueX(self) -> float: ...
    @property
    def AxisValueY(self) -> float: ...
    @property
    def Function(self) -> SimMultiValueFunction: ...
    @property
    def GraphName(self) -> str: ...
    @property
    def TargetParameter(self) -> SimBaseParameter: ...
    @TargetParameter.setter
    def TargetParameter(self, value: SimBaseParameter) -> SimBaseParameter: ...
    @property
    def ValueField(self) -> SimMultiValue: ...
    def Clone(self) -> SimParameterValueSource: ...
    def GetValue(self) -> typing.Any: ...
    def IsSamePointer(self, other: SimMultiValueParameterSource) -> bool: ...
    def SetFromParameters(self, axisValueX: float, axisValueY: float, axisValueZ: float, gs: str) -> None: ...


class SimMultiValueFunctionPointList(IEnumerable_1[SimPoint3D], INotifyCollectionChanged):
    def __init__(self, graph: SimMultiValueFunctionGraph, points: IEnumerable_1[SimPoint3D]) -> None: ...
    @property
    def Count(self) -> int: ...
    @property
    def Item(self) -> SimPoint3D: ...
    @Item.setter
    def Item(self, value: SimPoint3D) -> SimPoint3D: ...
    def Add(self, item: SimPoint3D) -> None: ...
    def GetEnumerator(self) -> IEnumerator_1[SimPoint3D]: ...
    def Insert(self, index: int, item: SimPoint3D) -> None: ...
    def RemoveAt(self, index: int) -> None: ...


class SimMultiValueParameterSource(SimParameterValueSource):
    @property
    def TargetParameter(self) -> SimBaseParameter: ...
    @TargetParameter.setter
    def TargetParameter(self, value: SimBaseParameter) -> SimBaseParameter: ...
    @property
    def ValueField(self) -> SimMultiValue: ...
    def CreateValuePointerParameters(self, user: SimUser) -> None: ...
    @abc.abstractmethod
    def GetValue(self) -> typing.Any: ...
    @abc.abstractmethod
    def IsSamePointer(self, other: SimMultiValueParameterSource) -> bool: ...
    @abc.abstractmethod
    def SetFromParameters(self, axisValueX: float, axisValueY: float, axisValueZ: float, graphName: str) -> None: ...


class SimMultiValueType(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Function : SimMultiValueType # 0
    Field3D : SimMultiValueType # 1
    BigTable : SimMultiValueType # 2




import typing, clr, abc
from System import Span_1, Array_1, ReadOnlySpan_1, Attribute

class AnsiStringMarshaller(abc.ABC):
    @staticmethod
    def ConvertToManaged(unmanaged: clr.Reference[int]) -> str: ...
    @staticmethod
    def ConvertToUnmanaged(managed: str) -> clr.Reference[int]: ...
    @staticmethod
    def Free(unmanaged: clr.Reference[int]) -> None: ...

    class ManagedToUnmanagedIn:
        @classmethod
        @property
        def BufferSize(cls) -> int: ...
        def Free(self) -> None: ...
        def FromManaged(self, managed: str, buffer: Span_1[int]) -> None: ...
        def ToUnmanaged(self) -> clr.Reference[int]: ...



class ArrayMarshaller_GenericClasses(abc.ABCMeta):
    Generic_ArrayMarshaller_GenericClasses_ArrayMarshaller_2_T = typing.TypeVar('Generic_ArrayMarshaller_GenericClasses_ArrayMarshaller_2_T')
    Generic_ArrayMarshaller_GenericClasses_ArrayMarshaller_2_TUnmanagedElement = typing.TypeVar('Generic_ArrayMarshaller_GenericClasses_ArrayMarshaller_2_TUnmanagedElement')
    def __getitem__(self, types : typing.Tuple[typing.Type[Generic_ArrayMarshaller_GenericClasses_ArrayMarshaller_2_T], typing.Type[Generic_ArrayMarshaller_GenericClasses_ArrayMarshaller_2_TUnmanagedElement]]) -> typing.Type[ArrayMarshaller_2[Generic_ArrayMarshaller_GenericClasses_ArrayMarshaller_2_T, Generic_ArrayMarshaller_GenericClasses_ArrayMarshaller_2_TUnmanagedElement]]: ...

ArrayMarshaller : ArrayMarshaller_GenericClasses

ArrayMarshaller_2_T = typing.TypeVar('ArrayMarshaller_2_T')
ArrayMarshaller_2_TUnmanagedElement = typing.TypeVar('ArrayMarshaller_2_TUnmanagedElement')
class ArrayMarshaller_2(typing.Generic[ArrayMarshaller_2_T, ArrayMarshaller_2_TUnmanagedElement], abc.ABC):
    @staticmethod
    def AllocateContainerForManagedElements(unmanaged: clr.Reference[ArrayMarshaller_2_TUnmanagedElement], numElements: int) -> Array_1[ArrayMarshaller_2_T]: ...
    @staticmethod
    def AllocateContainerForUnmanagedElements(managed: Array_1[ArrayMarshaller_2_T], numElements: clr.Reference[int]) -> clr.Reference[ArrayMarshaller_2_TUnmanagedElement]: ...
    @staticmethod
    def Free(unmanaged: clr.Reference[ArrayMarshaller_2_TUnmanagedElement]) -> None: ...
    @staticmethod
    def GetManagedValuesDestination(managed: Array_1[ArrayMarshaller_2_T]) -> Span_1[ArrayMarshaller_2_T]: ...
    @staticmethod
    def GetManagedValuesSource(managed: Array_1[ArrayMarshaller_2_T]) -> ReadOnlySpan_1[ArrayMarshaller_2_T]: ...
    @staticmethod
    def GetUnmanagedValuesDestination(unmanaged: clr.Reference[ArrayMarshaller_2_TUnmanagedElement], numElements: int) -> Span_1[ArrayMarshaller_2_TUnmanagedElement]: ...
    @staticmethod
    def GetUnmanagedValuesSource(unmanagedValue: clr.Reference[ArrayMarshaller_2_TUnmanagedElement], numElements: int) -> ReadOnlySpan_1[ArrayMarshaller_2_TUnmanagedElement]: ...

    ManagedToUnmanagedIn_GenericClasses_ArrayMarshaller_2_T = typing.TypeVar('ManagedToUnmanagedIn_GenericClasses_ArrayMarshaller_2_T')
    ManagedToUnmanagedIn_GenericClasses_ArrayMarshaller_2_TUnmanagedElement = typing.TypeVar('ManagedToUnmanagedIn_GenericClasses_ArrayMarshaller_2_TUnmanagedElement')
    class ManagedToUnmanagedIn_GenericClasses(typing.Generic[ManagedToUnmanagedIn_GenericClasses_ArrayMarshaller_2_T, ManagedToUnmanagedIn_GenericClasses_ArrayMarshaller_2_TUnmanagedElement], abc.ABCMeta):
        ManagedToUnmanagedIn_GenericClasses_ArrayMarshaller_2_T = ArrayMarshaller_2.ManagedToUnmanagedIn_GenericClasses_ArrayMarshaller_2_T
        ManagedToUnmanagedIn_GenericClasses_ArrayMarshaller_2_TUnmanagedElement = ArrayMarshaller_2.ManagedToUnmanagedIn_GenericClasses_ArrayMarshaller_2_TUnmanagedElement
        def __call__(self) -> ArrayMarshaller_2.ManagedToUnmanagedIn_2[ManagedToUnmanagedIn_GenericClasses_ArrayMarshaller_2_T, ManagedToUnmanagedIn_GenericClasses_ArrayMarshaller_2_TUnmanagedElement]: ...

    ManagedToUnmanagedIn : ManagedToUnmanagedIn_GenericClasses[ArrayMarshaller_2_T, ArrayMarshaller_2_TUnmanagedElement]

    ManagedToUnmanagedIn_2_T = typing.TypeVar('ManagedToUnmanagedIn_2_T')
    ManagedToUnmanagedIn_2_TUnmanagedElement = typing.TypeVar('ManagedToUnmanagedIn_2_TUnmanagedElement')
    class ManagedToUnmanagedIn_2(typing.Generic[ManagedToUnmanagedIn_2_T, ManagedToUnmanagedIn_2_TUnmanagedElement]):
        ManagedToUnmanagedIn_2_T = ArrayMarshaller_2.ManagedToUnmanagedIn_2_T
        ManagedToUnmanagedIn_2_TUnmanagedElement = ArrayMarshaller_2.ManagedToUnmanagedIn_2_TUnmanagedElement
        @classmethod
        @property
        def BufferSize(cls) -> int: ...
        def Free(self) -> None: ...
        def FromManaged(self, array: Array_1[ManagedToUnmanagedIn_2_T], buffer: Span_1[ManagedToUnmanagedIn_2_TUnmanagedElement]) -> None: ...
        def GetManagedValuesSource(self) -> ReadOnlySpan_1[ManagedToUnmanagedIn_2_T]: ...
        def GetUnmanagedValuesDestination(self) -> Span_1[ManagedToUnmanagedIn_2_TUnmanagedElement]: ...
        def ToUnmanaged(self) -> clr.Reference[ManagedToUnmanagedIn_2_TUnmanagedElement]: ...
        # Skipped GetPinnableReference due to it being static, abstract and generic.

        GetPinnableReference : GetPinnableReference_MethodGroup[ManagedToUnmanagedIn_2_T, ManagedToUnmanagedIn_2_TUnmanagedElement]
        GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T = typing.TypeVar('GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T')
        GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement = typing.TypeVar('GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement')
        class GetPinnableReference_MethodGroup(typing.Generic[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T, GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement]):
            GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T = ArrayMarshaller_2.ManagedToUnmanagedIn_2.GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T
            GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement = ArrayMarshaller_2.ManagedToUnmanagedIn_2.GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement
            @typing.overload
            def __call__(self) -> clr.Reference[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement]:...
            @typing.overload
            def __call__(self, array: Array_1[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T]) -> clr.Reference[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T]:...




class BStrStringMarshaller(abc.ABC):
    @staticmethod
    def ConvertToManaged(unmanaged: clr.Reference[int]) -> str: ...
    @staticmethod
    def ConvertToUnmanaged(managed: str) -> clr.Reference[int]: ...
    @staticmethod
    def Free(unmanaged: clr.Reference[int]) -> None: ...

    class ManagedToUnmanagedIn:
        @classmethod
        @property
        def BufferSize(cls) -> int: ...
        def Free(self) -> None: ...
        def FromManaged(self, managed: str, buffer: Span_1[int]) -> None: ...
        def ToUnmanaged(self) -> clr.Reference[int]: ...



class ContiguousCollectionMarshallerAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...


class CustomMarshallerAttribute(Attribute):
    def __init__(self, managedType: typing.Type[typing.Any], marshalMode: MarshalMode, marshallerType: typing.Type[typing.Any]) -> None: ...
    @property
    def ManagedType(self) -> typing.Type[typing.Any]: ...
    @property
    def MarshallerType(self) -> typing.Type[typing.Any]: ...
    @property
    def MarshalMode(self) -> MarshalMode: ...
    @property
    def TypeId(self) -> typing.Any: ...

    class GenericPlaceholder:
        pass



class MarshalMode(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Default : MarshalMode # 0
    ManagedToUnmanagedIn : MarshalMode # 1
    ManagedToUnmanagedRef : MarshalMode # 2
    ManagedToUnmanagedOut : MarshalMode # 3
    UnmanagedToManagedIn : MarshalMode # 4
    UnmanagedToManagedRef : MarshalMode # 5
    UnmanagedToManagedOut : MarshalMode # 6
    ElementIn : MarshalMode # 7
    ElementRef : MarshalMode # 8
    ElementOut : MarshalMode # 9


class MarshalUsingAttribute(Attribute):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, nativeType: typing.Type[typing.Any]) -> None: ...
    ReturnsCountValue : str
    @property
    def ConstantElementCount(self) -> int: ...
    @ConstantElementCount.setter
    def ConstantElementCount(self, value: int) -> int: ...
    @property
    def CountElementName(self) -> str: ...
    @CountElementName.setter
    def CountElementName(self, value: str) -> str: ...
    @property
    def ElementIndirectionDepth(self) -> int: ...
    @ElementIndirectionDepth.setter
    def ElementIndirectionDepth(self, value: int) -> int: ...
    @property
    def NativeType(self) -> typing.Type[typing.Any]: ...
    @property
    def TypeId(self) -> typing.Any: ...


class NativeMarshallingAttribute(Attribute):
    def __init__(self, nativeType: typing.Type[typing.Any]) -> None: ...
    @property
    def NativeType(self) -> typing.Type[typing.Any]: ...
    @property
    def TypeId(self) -> typing.Any: ...


class PointerArrayMarshaller_GenericClasses(abc.ABCMeta):
    Generic_PointerArrayMarshaller_GenericClasses_PointerArrayMarshaller_2_T = typing.TypeVar('Generic_PointerArrayMarshaller_GenericClasses_PointerArrayMarshaller_2_T')
    Generic_PointerArrayMarshaller_GenericClasses_PointerArrayMarshaller_2_TUnmanagedElement = typing.TypeVar('Generic_PointerArrayMarshaller_GenericClasses_PointerArrayMarshaller_2_TUnmanagedElement')
    def __getitem__(self, types : typing.Tuple[typing.Type[Generic_PointerArrayMarshaller_GenericClasses_PointerArrayMarshaller_2_T], typing.Type[Generic_PointerArrayMarshaller_GenericClasses_PointerArrayMarshaller_2_TUnmanagedElement]]) -> typing.Type[PointerArrayMarshaller_2[Generic_PointerArrayMarshaller_GenericClasses_PointerArrayMarshaller_2_T, Generic_PointerArrayMarshaller_GenericClasses_PointerArrayMarshaller_2_TUnmanagedElement]]: ...

PointerArrayMarshaller : PointerArrayMarshaller_GenericClasses

PointerArrayMarshaller_2_T = typing.TypeVar('PointerArrayMarshaller_2_T')
PointerArrayMarshaller_2_TUnmanagedElement = typing.TypeVar('PointerArrayMarshaller_2_TUnmanagedElement')
class PointerArrayMarshaller_2(typing.Generic[PointerArrayMarshaller_2_T, PointerArrayMarshaller_2_TUnmanagedElement], abc.ABC):
    @staticmethod
    def AllocateContainerForManagedElements(unmanaged: clr.Reference[PointerArrayMarshaller_2_TUnmanagedElement], numElements: int) -> Array_1[clr.Reference[PointerArrayMarshaller_2_T]]: ...
    @staticmethod
    def AllocateContainerForUnmanagedElements(managed: Array_1[clr.Reference[PointerArrayMarshaller_2_T]], numElements: clr.Reference[int]) -> clr.Reference[PointerArrayMarshaller_2_TUnmanagedElement]: ...
    @staticmethod
    def Free(unmanaged: clr.Reference[PointerArrayMarshaller_2_TUnmanagedElement]) -> None: ...
    @staticmethod
    def GetManagedValuesDestination(managed: Array_1[clr.Reference[PointerArrayMarshaller_2_T]]) -> Span_1[int]: ...
    @staticmethod
    def GetManagedValuesSource(managed: Array_1[clr.Reference[PointerArrayMarshaller_2_T]]) -> ReadOnlySpan_1[int]: ...
    @staticmethod
    def GetUnmanagedValuesDestination(unmanaged: clr.Reference[PointerArrayMarshaller_2_TUnmanagedElement], numElements: int) -> Span_1[PointerArrayMarshaller_2_TUnmanagedElement]: ...
    @staticmethod
    def GetUnmanagedValuesSource(unmanagedValue: clr.Reference[PointerArrayMarshaller_2_TUnmanagedElement], numElements: int) -> ReadOnlySpan_1[PointerArrayMarshaller_2_TUnmanagedElement]: ...

    ManagedToUnmanagedIn_GenericClasses_PointerArrayMarshaller_2_T = typing.TypeVar('ManagedToUnmanagedIn_GenericClasses_PointerArrayMarshaller_2_T')
    ManagedToUnmanagedIn_GenericClasses_PointerArrayMarshaller_2_TUnmanagedElement = typing.TypeVar('ManagedToUnmanagedIn_GenericClasses_PointerArrayMarshaller_2_TUnmanagedElement')
    class ManagedToUnmanagedIn_GenericClasses(typing.Generic[ManagedToUnmanagedIn_GenericClasses_PointerArrayMarshaller_2_T, ManagedToUnmanagedIn_GenericClasses_PointerArrayMarshaller_2_TUnmanagedElement], abc.ABCMeta):
        ManagedToUnmanagedIn_GenericClasses_PointerArrayMarshaller_2_T = PointerArrayMarshaller_2.ManagedToUnmanagedIn_GenericClasses_PointerArrayMarshaller_2_T
        ManagedToUnmanagedIn_GenericClasses_PointerArrayMarshaller_2_TUnmanagedElement = PointerArrayMarshaller_2.ManagedToUnmanagedIn_GenericClasses_PointerArrayMarshaller_2_TUnmanagedElement
        def __call__(self) -> PointerArrayMarshaller_2.ManagedToUnmanagedIn_2[ManagedToUnmanagedIn_GenericClasses_PointerArrayMarshaller_2_T, ManagedToUnmanagedIn_GenericClasses_PointerArrayMarshaller_2_TUnmanagedElement]: ...

    ManagedToUnmanagedIn : ManagedToUnmanagedIn_GenericClasses[PointerArrayMarshaller_2_T, PointerArrayMarshaller_2_TUnmanagedElement]

    ManagedToUnmanagedIn_2_T = typing.TypeVar('ManagedToUnmanagedIn_2_T')
    ManagedToUnmanagedIn_2_TUnmanagedElement = typing.TypeVar('ManagedToUnmanagedIn_2_TUnmanagedElement')
    class ManagedToUnmanagedIn_2(typing.Generic[ManagedToUnmanagedIn_2_T, ManagedToUnmanagedIn_2_TUnmanagedElement]):
        ManagedToUnmanagedIn_2_T = PointerArrayMarshaller_2.ManagedToUnmanagedIn_2_T
        ManagedToUnmanagedIn_2_TUnmanagedElement = PointerArrayMarshaller_2.ManagedToUnmanagedIn_2_TUnmanagedElement
        @classmethod
        @property
        def BufferSize(cls) -> int: ...
        def Free(self) -> None: ...
        def FromManaged(self, array: Array_1[clr.Reference[ManagedToUnmanagedIn_2_T]], buffer: Span_1[ManagedToUnmanagedIn_2_TUnmanagedElement]) -> None: ...
        def GetManagedValuesSource(self) -> ReadOnlySpan_1[int]: ...
        def GetUnmanagedValuesDestination(self) -> Span_1[ManagedToUnmanagedIn_2_TUnmanagedElement]: ...
        def ToUnmanaged(self) -> clr.Reference[ManagedToUnmanagedIn_2_TUnmanagedElement]: ...
        # Skipped GetPinnableReference due to it being static, abstract and generic.

        GetPinnableReference : GetPinnableReference_MethodGroup[ManagedToUnmanagedIn_2_T, ManagedToUnmanagedIn_2_TUnmanagedElement]
        GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T = typing.TypeVar('GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T')
        GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement = typing.TypeVar('GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement')
        class GetPinnableReference_MethodGroup(typing.Generic[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T, GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement]):
            GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T = PointerArrayMarshaller_2.ManagedToUnmanagedIn_2.GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T
            GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement = PointerArrayMarshaller_2.ManagedToUnmanagedIn_2.GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement
            @typing.overload
            def __call__(self) -> clr.Reference[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement]:...
            @typing.overload
            def __call__(self, array: Array_1[clr.Reference[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T]]) -> clr.Reference[int]:...




class ReadOnlySpanMarshaller_GenericClasses(abc.ABCMeta):
    Generic_ReadOnlySpanMarshaller_GenericClasses_ReadOnlySpanMarshaller_2_T = typing.TypeVar('Generic_ReadOnlySpanMarshaller_GenericClasses_ReadOnlySpanMarshaller_2_T')
    Generic_ReadOnlySpanMarshaller_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement = typing.TypeVar('Generic_ReadOnlySpanMarshaller_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement')
    def __getitem__(self, types : typing.Tuple[typing.Type[Generic_ReadOnlySpanMarshaller_GenericClasses_ReadOnlySpanMarshaller_2_T], typing.Type[Generic_ReadOnlySpanMarshaller_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement]]) -> typing.Type[ReadOnlySpanMarshaller_2[Generic_ReadOnlySpanMarshaller_GenericClasses_ReadOnlySpanMarshaller_2_T, Generic_ReadOnlySpanMarshaller_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement]]: ...

ReadOnlySpanMarshaller : ReadOnlySpanMarshaller_GenericClasses

ReadOnlySpanMarshaller_2_T = typing.TypeVar('ReadOnlySpanMarshaller_2_T')
ReadOnlySpanMarshaller_2_TUnmanagedElement = typing.TypeVar('ReadOnlySpanMarshaller_2_TUnmanagedElement')
class ReadOnlySpanMarshaller_2(typing.Generic[ReadOnlySpanMarshaller_2_T, ReadOnlySpanMarshaller_2_TUnmanagedElement], abc.ABC):

    ManagedToUnmanagedIn_GenericClasses_ReadOnlySpanMarshaller_2_T = typing.TypeVar('ManagedToUnmanagedIn_GenericClasses_ReadOnlySpanMarshaller_2_T')
    ManagedToUnmanagedIn_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement = typing.TypeVar('ManagedToUnmanagedIn_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement')
    class ManagedToUnmanagedIn_GenericClasses(typing.Generic[ManagedToUnmanagedIn_GenericClasses_ReadOnlySpanMarshaller_2_T, ManagedToUnmanagedIn_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement], abc.ABCMeta):
        ManagedToUnmanagedIn_GenericClasses_ReadOnlySpanMarshaller_2_T = ReadOnlySpanMarshaller_2.ManagedToUnmanagedIn_GenericClasses_ReadOnlySpanMarshaller_2_T
        ManagedToUnmanagedIn_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement = ReadOnlySpanMarshaller_2.ManagedToUnmanagedIn_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement
        def __call__(self) -> ReadOnlySpanMarshaller_2.ManagedToUnmanagedIn_2[ManagedToUnmanagedIn_GenericClasses_ReadOnlySpanMarshaller_2_T, ManagedToUnmanagedIn_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement]: ...

    ManagedToUnmanagedIn : ManagedToUnmanagedIn_GenericClasses[ReadOnlySpanMarshaller_2_T, ReadOnlySpanMarshaller_2_TUnmanagedElement]

    ManagedToUnmanagedIn_2_T = typing.TypeVar('ManagedToUnmanagedIn_2_T')
    ManagedToUnmanagedIn_2_TUnmanagedElement = typing.TypeVar('ManagedToUnmanagedIn_2_TUnmanagedElement')
    class ManagedToUnmanagedIn_2(typing.Generic[ManagedToUnmanagedIn_2_T, ManagedToUnmanagedIn_2_TUnmanagedElement]):
        ManagedToUnmanagedIn_2_T = ReadOnlySpanMarshaller_2.ManagedToUnmanagedIn_2_T
        ManagedToUnmanagedIn_2_TUnmanagedElement = ReadOnlySpanMarshaller_2.ManagedToUnmanagedIn_2_TUnmanagedElement
        @classmethod
        @property
        def BufferSize(cls) -> int: ...
        def Free(self) -> None: ...
        def FromManaged(self, managed: ReadOnlySpan_1[ManagedToUnmanagedIn_2_T], buffer: Span_1[ManagedToUnmanagedIn_2_TUnmanagedElement]) -> None: ...
        def GetManagedValuesSource(self) -> ReadOnlySpan_1[ManagedToUnmanagedIn_2_T]: ...
        def GetUnmanagedValuesDestination(self) -> Span_1[ManagedToUnmanagedIn_2_TUnmanagedElement]: ...
        def ToUnmanaged(self) -> clr.Reference[ManagedToUnmanagedIn_2_TUnmanagedElement]: ...
        # Skipped GetPinnableReference due to it being static, abstract and generic.

        GetPinnableReference : GetPinnableReference_MethodGroup[ManagedToUnmanagedIn_2_T, ManagedToUnmanagedIn_2_TUnmanagedElement]
        GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T = typing.TypeVar('GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T')
        GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement = typing.TypeVar('GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement')
        class GetPinnableReference_MethodGroup(typing.Generic[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T, GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement]):
            GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T = ReadOnlySpanMarshaller_2.ManagedToUnmanagedIn_2.GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T
            GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement = ReadOnlySpanMarshaller_2.ManagedToUnmanagedIn_2.GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement
            @typing.overload
            def __call__(self) -> clr.Reference[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement]:...
            @typing.overload
            def __call__(self, managed: ReadOnlySpan_1[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T]) -> clr.Reference[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T]:...



    UnmanagedToManagedOut_GenericClasses_ReadOnlySpanMarshaller_2_T = typing.TypeVar('UnmanagedToManagedOut_GenericClasses_ReadOnlySpanMarshaller_2_T')
    UnmanagedToManagedOut_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement = typing.TypeVar('UnmanagedToManagedOut_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement')
    class UnmanagedToManagedOut_GenericClasses(typing.Generic[UnmanagedToManagedOut_GenericClasses_ReadOnlySpanMarshaller_2_T, UnmanagedToManagedOut_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement], abc.ABCMeta):
        UnmanagedToManagedOut_GenericClasses_ReadOnlySpanMarshaller_2_T = ReadOnlySpanMarshaller_2.UnmanagedToManagedOut_GenericClasses_ReadOnlySpanMarshaller_2_T
        UnmanagedToManagedOut_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement = ReadOnlySpanMarshaller_2.UnmanagedToManagedOut_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement
        def __call__(self) -> ReadOnlySpanMarshaller_2.UnmanagedToManagedOut_2[UnmanagedToManagedOut_GenericClasses_ReadOnlySpanMarshaller_2_T, UnmanagedToManagedOut_GenericClasses_ReadOnlySpanMarshaller_2_TUnmanagedElement]: ...

    UnmanagedToManagedOut : UnmanagedToManagedOut_GenericClasses[ReadOnlySpanMarshaller_2_T, ReadOnlySpanMarshaller_2_TUnmanagedElement]

    UnmanagedToManagedOut_2_T = typing.TypeVar('UnmanagedToManagedOut_2_T')
    UnmanagedToManagedOut_2_TUnmanagedElement = typing.TypeVar('UnmanagedToManagedOut_2_TUnmanagedElement')
    class UnmanagedToManagedOut_2(typing.Generic[UnmanagedToManagedOut_2_T, UnmanagedToManagedOut_2_TUnmanagedElement], abc.ABC):
        UnmanagedToManagedOut_2_T = ReadOnlySpanMarshaller_2.UnmanagedToManagedOut_2_T
        UnmanagedToManagedOut_2_TUnmanagedElement = ReadOnlySpanMarshaller_2.UnmanagedToManagedOut_2_TUnmanagedElement
        @staticmethod
        def AllocateContainerForUnmanagedElements(managed: ReadOnlySpan_1[UnmanagedToManagedOut_2_T], numElements: clr.Reference[int]) -> clr.Reference[UnmanagedToManagedOut_2_TUnmanagedElement]: ...
        @staticmethod
        def GetManagedValuesSource(managed: ReadOnlySpan_1[UnmanagedToManagedOut_2_T]) -> ReadOnlySpan_1[UnmanagedToManagedOut_2_T]: ...
        @staticmethod
        def GetUnmanagedValuesDestination(unmanaged: clr.Reference[UnmanagedToManagedOut_2_TUnmanagedElement], numElements: int) -> Span_1[UnmanagedToManagedOut_2_TUnmanagedElement]: ...



class SafeHandleMarshaller_GenericClasses(abc.ABCMeta):
    Generic_SafeHandleMarshaller_GenericClasses_SafeHandleMarshaller_1_T = typing.TypeVar('Generic_SafeHandleMarshaller_GenericClasses_SafeHandleMarshaller_1_T')
    def __getitem__(self, types : typing.Type[Generic_SafeHandleMarshaller_GenericClasses_SafeHandleMarshaller_1_T]) -> typing.Type[SafeHandleMarshaller_1[Generic_SafeHandleMarshaller_GenericClasses_SafeHandleMarshaller_1_T]]: ...

SafeHandleMarshaller : SafeHandleMarshaller_GenericClasses

SafeHandleMarshaller_1_T = typing.TypeVar('SafeHandleMarshaller_1_T')
class SafeHandleMarshaller_1(typing.Generic[SafeHandleMarshaller_1_T], abc.ABC):

    ManagedToUnmanagedIn_GenericClasses_SafeHandleMarshaller_1_T = typing.TypeVar('ManagedToUnmanagedIn_GenericClasses_SafeHandleMarshaller_1_T')
    class ManagedToUnmanagedIn_GenericClasses(typing.Generic[ManagedToUnmanagedIn_GenericClasses_SafeHandleMarshaller_1_T], abc.ABCMeta):
        ManagedToUnmanagedIn_GenericClasses_SafeHandleMarshaller_1_T = SafeHandleMarshaller_1.ManagedToUnmanagedIn_GenericClasses_SafeHandleMarshaller_1_T
        def __call__(self) -> SafeHandleMarshaller_1.ManagedToUnmanagedIn_1[ManagedToUnmanagedIn_GenericClasses_SafeHandleMarshaller_1_T]: ...

    ManagedToUnmanagedIn : ManagedToUnmanagedIn_GenericClasses[SafeHandleMarshaller_1_T]

    ManagedToUnmanagedIn_1_T = typing.TypeVar('ManagedToUnmanagedIn_1_T')
    class ManagedToUnmanagedIn_1(typing.Generic[ManagedToUnmanagedIn_1_T]):
        ManagedToUnmanagedIn_1_T = SafeHandleMarshaller_1.ManagedToUnmanagedIn_1_T
        def Free(self) -> None: ...
        def FromManaged(self, handle: ManagedToUnmanagedIn_1_T) -> None: ...
        def ToUnmanaged(self) -> int: ...


    ManagedToUnmanagedOut_GenericClasses_SafeHandleMarshaller_1_T = typing.TypeVar('ManagedToUnmanagedOut_GenericClasses_SafeHandleMarshaller_1_T')
    class ManagedToUnmanagedOut_GenericClasses(typing.Generic[ManagedToUnmanagedOut_GenericClasses_SafeHandleMarshaller_1_T], abc.ABCMeta):
        ManagedToUnmanagedOut_GenericClasses_SafeHandleMarshaller_1_T = SafeHandleMarshaller_1.ManagedToUnmanagedOut_GenericClasses_SafeHandleMarshaller_1_T
        def __call__(self) -> SafeHandleMarshaller_1.ManagedToUnmanagedOut_1[ManagedToUnmanagedOut_GenericClasses_SafeHandleMarshaller_1_T]: ...

    ManagedToUnmanagedOut : ManagedToUnmanagedOut_GenericClasses[SafeHandleMarshaller_1_T]

    ManagedToUnmanagedOut_1_T = typing.TypeVar('ManagedToUnmanagedOut_1_T')
    class ManagedToUnmanagedOut_1(typing.Generic[ManagedToUnmanagedOut_1_T]):
        ManagedToUnmanagedOut_1_T = SafeHandleMarshaller_1.ManagedToUnmanagedOut_1_T
        def __init__(self) -> None: ...
        def Free(self) -> None: ...
        def FromUnmanaged(self, value: int) -> None: ...
        def ToManaged(self) -> ManagedToUnmanagedOut_1_T: ...


    ManagedToUnmanagedRef_GenericClasses_SafeHandleMarshaller_1_T = typing.TypeVar('ManagedToUnmanagedRef_GenericClasses_SafeHandleMarshaller_1_T')
    class ManagedToUnmanagedRef_GenericClasses(typing.Generic[ManagedToUnmanagedRef_GenericClasses_SafeHandleMarshaller_1_T], abc.ABCMeta):
        ManagedToUnmanagedRef_GenericClasses_SafeHandleMarshaller_1_T = SafeHandleMarshaller_1.ManagedToUnmanagedRef_GenericClasses_SafeHandleMarshaller_1_T
        def __call__(self) -> SafeHandleMarshaller_1.ManagedToUnmanagedRef_1[ManagedToUnmanagedRef_GenericClasses_SafeHandleMarshaller_1_T]: ...

    ManagedToUnmanagedRef : ManagedToUnmanagedRef_GenericClasses[SafeHandleMarshaller_1_T]

    ManagedToUnmanagedRef_1_T = typing.TypeVar('ManagedToUnmanagedRef_1_T')
    class ManagedToUnmanagedRef_1(typing.Generic[ManagedToUnmanagedRef_1_T]):
        ManagedToUnmanagedRef_1_T = SafeHandleMarshaller_1.ManagedToUnmanagedRef_1_T
        def __init__(self) -> None: ...
        def Free(self) -> None: ...
        def FromManaged(self, handle: ManagedToUnmanagedRef_1_T) -> None: ...
        def FromUnmanaged(self, value: int) -> None: ...
        def OnInvoked(self) -> None: ...
        def ToManagedFinally(self) -> ManagedToUnmanagedRef_1_T: ...
        def ToUnmanaged(self) -> int: ...



class SpanMarshaller_GenericClasses(abc.ABCMeta):
    Generic_SpanMarshaller_GenericClasses_SpanMarshaller_2_T = typing.TypeVar('Generic_SpanMarshaller_GenericClasses_SpanMarshaller_2_T')
    Generic_SpanMarshaller_GenericClasses_SpanMarshaller_2_TUnmanagedElement = typing.TypeVar('Generic_SpanMarshaller_GenericClasses_SpanMarshaller_2_TUnmanagedElement')
    def __getitem__(self, types : typing.Tuple[typing.Type[Generic_SpanMarshaller_GenericClasses_SpanMarshaller_2_T], typing.Type[Generic_SpanMarshaller_GenericClasses_SpanMarshaller_2_TUnmanagedElement]]) -> typing.Type[SpanMarshaller_2[Generic_SpanMarshaller_GenericClasses_SpanMarshaller_2_T, Generic_SpanMarshaller_GenericClasses_SpanMarshaller_2_TUnmanagedElement]]: ...

SpanMarshaller : SpanMarshaller_GenericClasses

SpanMarshaller_2_T = typing.TypeVar('SpanMarshaller_2_T')
SpanMarshaller_2_TUnmanagedElement = typing.TypeVar('SpanMarshaller_2_TUnmanagedElement')
class SpanMarshaller_2(typing.Generic[SpanMarshaller_2_T, SpanMarshaller_2_TUnmanagedElement], abc.ABC):
    @staticmethod
    def AllocateContainerForManagedElements(unmanaged: clr.Reference[SpanMarshaller_2_TUnmanagedElement], numElements: int) -> Span_1[SpanMarshaller_2_T]: ...
    @staticmethod
    def AllocateContainerForUnmanagedElements(managed: Span_1[SpanMarshaller_2_T], numElements: clr.Reference[int]) -> clr.Reference[SpanMarshaller_2_TUnmanagedElement]: ...
    @staticmethod
    def Free(unmanaged: clr.Reference[SpanMarshaller_2_TUnmanagedElement]) -> None: ...
    @staticmethod
    def GetManagedValuesDestination(managed: Span_1[SpanMarshaller_2_T]) -> Span_1[SpanMarshaller_2_T]: ...
    @staticmethod
    def GetManagedValuesSource(managed: Span_1[SpanMarshaller_2_T]) -> ReadOnlySpan_1[SpanMarshaller_2_T]: ...
    @staticmethod
    def GetUnmanagedValuesDestination(unmanaged: clr.Reference[SpanMarshaller_2_TUnmanagedElement], numElements: int) -> Span_1[SpanMarshaller_2_TUnmanagedElement]: ...
    @staticmethod
    def GetUnmanagedValuesSource(unmanaged: clr.Reference[SpanMarshaller_2_TUnmanagedElement], numElements: int) -> ReadOnlySpan_1[SpanMarshaller_2_TUnmanagedElement]: ...

    ManagedToUnmanagedIn_GenericClasses_SpanMarshaller_2_T = typing.TypeVar('ManagedToUnmanagedIn_GenericClasses_SpanMarshaller_2_T')
    ManagedToUnmanagedIn_GenericClasses_SpanMarshaller_2_TUnmanagedElement = typing.TypeVar('ManagedToUnmanagedIn_GenericClasses_SpanMarshaller_2_TUnmanagedElement')
    class ManagedToUnmanagedIn_GenericClasses(typing.Generic[ManagedToUnmanagedIn_GenericClasses_SpanMarshaller_2_T, ManagedToUnmanagedIn_GenericClasses_SpanMarshaller_2_TUnmanagedElement], abc.ABCMeta):
        ManagedToUnmanagedIn_GenericClasses_SpanMarshaller_2_T = SpanMarshaller_2.ManagedToUnmanagedIn_GenericClasses_SpanMarshaller_2_T
        ManagedToUnmanagedIn_GenericClasses_SpanMarshaller_2_TUnmanagedElement = SpanMarshaller_2.ManagedToUnmanagedIn_GenericClasses_SpanMarshaller_2_TUnmanagedElement
        def __call__(self) -> SpanMarshaller_2.ManagedToUnmanagedIn_2[ManagedToUnmanagedIn_GenericClasses_SpanMarshaller_2_T, ManagedToUnmanagedIn_GenericClasses_SpanMarshaller_2_TUnmanagedElement]: ...

    ManagedToUnmanagedIn : ManagedToUnmanagedIn_GenericClasses[SpanMarshaller_2_T, SpanMarshaller_2_TUnmanagedElement]

    ManagedToUnmanagedIn_2_T = typing.TypeVar('ManagedToUnmanagedIn_2_T')
    ManagedToUnmanagedIn_2_TUnmanagedElement = typing.TypeVar('ManagedToUnmanagedIn_2_TUnmanagedElement')
    class ManagedToUnmanagedIn_2(typing.Generic[ManagedToUnmanagedIn_2_T, ManagedToUnmanagedIn_2_TUnmanagedElement]):
        ManagedToUnmanagedIn_2_T = SpanMarshaller_2.ManagedToUnmanagedIn_2_T
        ManagedToUnmanagedIn_2_TUnmanagedElement = SpanMarshaller_2.ManagedToUnmanagedIn_2_TUnmanagedElement
        @classmethod
        @property
        def BufferSize(cls) -> int: ...
        def Free(self) -> None: ...
        def FromManaged(self, managed: Span_1[ManagedToUnmanagedIn_2_T], buffer: Span_1[ManagedToUnmanagedIn_2_TUnmanagedElement]) -> None: ...
        def GetManagedValuesSource(self) -> ReadOnlySpan_1[ManagedToUnmanagedIn_2_T]: ...
        def GetUnmanagedValuesDestination(self) -> Span_1[ManagedToUnmanagedIn_2_TUnmanagedElement]: ...
        def ToUnmanaged(self) -> clr.Reference[ManagedToUnmanagedIn_2_TUnmanagedElement]: ...
        # Skipped GetPinnableReference due to it being static, abstract and generic.

        GetPinnableReference : GetPinnableReference_MethodGroup[ManagedToUnmanagedIn_2_T, ManagedToUnmanagedIn_2_TUnmanagedElement]
        GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T = typing.TypeVar('GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T')
        GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement = typing.TypeVar('GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement')
        class GetPinnableReference_MethodGroup(typing.Generic[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T, GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement]):
            GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T = SpanMarshaller_2.ManagedToUnmanagedIn_2.GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T
            GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement = SpanMarshaller_2.ManagedToUnmanagedIn_2.GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement
            @typing.overload
            def __call__(self) -> clr.Reference[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_TUnmanagedElement]:...
            @typing.overload
            def __call__(self, managed: Span_1[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T]) -> clr.Reference[GetPinnableReference_MethodGroup_ManagedToUnmanagedIn_2_T]:...




class Utf16StringMarshaller(abc.ABC):
    @staticmethod
    def ConvertToManaged(unmanaged: clr.Reference[int]) -> str: ...
    @staticmethod
    def ConvertToUnmanaged(managed: str) -> clr.Reference[int]: ...
    @staticmethod
    def Free(unmanaged: clr.Reference[int]) -> None: ...
    @staticmethod
    def GetPinnableReference(str: str) -> clr.Reference[str]: ...


class Utf8StringMarshaller(abc.ABC):
    @staticmethod
    def ConvertToManaged(unmanaged: clr.Reference[int]) -> str: ...
    @staticmethod
    def ConvertToUnmanaged(managed: str) -> clr.Reference[int]: ...
    @staticmethod
    def Free(unmanaged: clr.Reference[int]) -> None: ...

    class ManagedToUnmanagedIn:
        @classmethod
        @property
        def BufferSize(cls) -> int: ...
        def Free(self) -> None: ...
        def FromManaged(self, managed: str, buffer: Span_1[int]) -> None: ...
        def ToUnmanaged(self) -> clr.Reference[int]: ...


